# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from unittest.mock import Mock

from superdesk.publish.transmitters.email import EmailPublishService
from superdesk.tests import TestCase
from superdesk.publish import init_app


class MockMediaFS:
    """Simple Mocked Media FS"""

    def __init__(self):
        self.get = Mock()
        self.get.side_effect = self._get
        self.read = Mock()
        self.read.side_effect = self._read

    name = "filename"
    content_type = "image/jpeg"

    def _get(self, id, resource):
        return self

    def _read(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", "IPTC-PhotometadataRef-Std2017.1.jpg"))
        with open(fixture, "rb") as f:
            return f.read()


class MockMail:
    def __init__(self):
        self.send = Mock()
        self.send.side_effect = self._send

    def _send(self, message):
        if message.subject != "Test Subject":
            raise ValueError("Unexpected Subjet")
        if len(message.attachments) != 1:
            raise Exception("Wrong number of attachments")


class EmailPublishServiceTest(TestCase):
    filename = "IPTC-PhotometadataRef-Std2017.1.jpg"

    def setUp(self):
        init_app(self.app)
        self._media = self.app.media
        self.app.media = MockMediaFS()
        self._mail = self.app.mail
        self.app.mail = MockMail()

    def tearDown(self):
        self.app.mail = self._mail
        self.app.media = self._media

    def test_attachment(self):
        queue_item = {
            "item_id": "123",
            "destination": {
                "delivery_type": "email",
                "config": {
                    "media_cid": "MainImage",
                    "media_rendition": "viewImage",
                    "recipients": "a@b.c.d",
                    "attach_media": True,
                    "watermark": True,
                },
                "name": "Email",
                "format": "Email",
            },
            "formatted_item": '{"message_text": "Test", "message_html": "<p>Test</p>", '
            '"message_subject": "Test Subject", "renditions": {"viewImage": {"media": "1234"}}}',
        }

        transmitter = EmailPublishService()
        transmitter._transmit(queue_item=queue_item, subscriber={})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from eve.utils import config
from flask import request, current_app as app

from superdesk import get_resource_service, Service
from superdesk.metadata.item import GUID_TAG
from superdesk.resource import Resource
from apps.archive import ArchiveSpikeService
from apps.archive.common import ITEM_UNLINK
from apps.auth import get_user
from superdesk.metadata.utils import item_url, generate_guid
from apps.archive.archive import SOURCE as ARCHIVE
from superdesk.errors import SuperdeskApiError
from superdesk.notification import push_notification
import logging
from flask_babel import _

logger = logging.getLogger(__name__)


class ArchiveLinkResource(Resource):
    endpoint_name = "archive_link"
    resource_title = endpoint_name

    schema = {
        "link_id": Resource.rel("archive", embeddable=False, type="string", nullable=True, required=False),
        "desk": Resource.rel("desks", embeddable=False, nullable=True, required=False),
    }

    url = "archive/<{0}:target_id>/link".format(item_url)

    resource_methods = ["DELETE"]
    item_methods = []


class ArchiveLinkService(Service):
    def delete(self, lookup):
        target_id = request.view_args["target_id"]
        archive_service = get_resource_service(ARCHIVE)
        target = archive_service.find_one(req=None, _id=target_id)
        updates = {}

        if target.get("rewrite_of"):
            # remove the rewrite info
            ArchiveSpikeService().update_rewrite(target)

        if not target.get("rewrite_of"):
            # there is nothing to do
            raise SuperdeskApiError.badRequestError(_("Only updates can be unlinked!"))

        if target.get("rewrite_of"):
            updates["rewrite_of"] = None

        if target.get("anpa_take_key"):
            updates["anpa_take_key"] = None

        if target.get("rewrite_sequence"):
            updates["rewrite_sequence"] = None

        if target.get("sequence"):
            updates["sequence"] = None

        updates["event_id"] = generate_guid(type=GUID_TAG)

        archive_service.system_update(target_id, updates, target)
        user = get_user(required=True)
        push_notification("item:unlink", item=target_id, user=str(user.get(config.ID_FIELD)))
        app.on_archive_item_updated(updates, target, ITEM_UNLINK)



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.feed_parsers.nitf import NITFFeedParser, SkipValue
from superdesk.io.registry import register_feed_parser
import re
from lxml import etree
import html


class PAFeedParser(NITFFeedParser):
    """
    NITF Parser extension for Press Association, it maps the category meta tag to an anpa category
    """

    NAME = "pa_nitf"

    label = "PA NITF"

    def _category_mapping(self, elem):
        """Map the category supplied by PA to a best guess anpa_category in the system.

        There is a special case for the incomming category 'HHH' is to check the doc-scope for SHOWBIZ, if present the
        category is set to Entertainment

        :param elem:
        :return: anpa category list qcode
        """
        if elem.get("content") is not None:
            category = elem.get("content")[:1].upper()
            if category in {"S", "R", "F"}:
                return [{"qcode": "S"}]
            if category == "Z":
                return [{"qcode": "V"}]
            if category == "H":
                if self.xml.find("head/docdata/doc-scope[@scope='SHOWBIZ']") is not None:
                    return [{"qcode": "E"}]
        return [{"qcode": "I"}]

    def get_content(self, xml):
        """Get the body content of the item.

        Remove the child tags of the p tags such as location, person etc. These have no meaning in the editor at the
        moment. Also handle the encoding, for example the £ symbol in the body text.

        :param xml:
        :return:
        """
        elements = []
        for elem in xml.find("body/body.content"):
            text = etree.tostring(elem, encoding="unicode", method="text")
            elements.append("<p>{}</p>\n".format(html.escape(text)))
        content = "".join(elements)
        if self.get_anpa_format(xml) == "t":
            if not content.startswith("<pre>"):
                # convert content to text in a pre tag
                content = "<pre>{}</pre>".format(self.parse_to_preformatted(content))
            else:
                content = self.parse_to_preformatted(content)
        return content

    def get_headline(self, xml):
        """Return the headline if available if not then return the slugline (title)

        :param xml:
        :return:
        """
        if xml.find("body/body.head/hedline/hl1") is not None:
            return xml.find("body/body.head/hedline/hl1").text
        else:
            if xml.find("head/title") is not None:
                return self._get_slugline(xml.find("head/title"))
        raise SkipValue()

    def _get_slugline(self, elem):
        """Capitalize the first word of the slugline (Removing any leading digits's).

        :param elem:
        :return:
        """
        # Remove any leading numbers and split to list of words
        sluglineList = re.sub(r"^[\d.]+\W+", "", elem.text).split(" ")
        slugline = sluglineList[0].capitalize()
        if len(sluglineList) > 1:
            slugline = "{} {}".format(slugline, " ".join(sluglineList[1:]))
        return slugline

    def _get_pubstatus(self, elem):
        """Mark anything that is embargoed as usable, the editorial note still describes the embargo.

        :param elem:
        :return:
        """
        return "usable" if elem.attrib["management-status"] == "embargoed" else elem.attrib["management-status"]

    def __init__(self):
        self.MAPPING = {
            "anpa_category": {"xpath": "head/meta[@name='category']", "filter": self._category_mapping},
            "slugline": {"xpath": "head/title", "filter": self._get_slugline},
            "pubstatus": {"xpath": "head/docdata", "filter": self._get_pubstatus},
        }
        super().__init__()

    def parse(self, xml, provider=None):
        self.xml = xml
        return super().parse(xml, provider=provider)


register_feed_parser(PAFeedParser.NAME, PAFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk import get_resource_service
from superdesk.tests import TestCase
from nose.tools import assert_raises


class SequencesTestCase(TestCase):
    def setUp(self):
        with self.app.app_context():
            self.service = get_resource_service("sequences")
        self.min_seq_number = 1
        self.max_seq_number = 10

    def test_empty_sequence_name_fails(self):
        with self.app.app_context():
            with assert_raises(KeyError):
                self.service.get_next_sequence_number(None)
                self.service.get_next_sequence_number("")

    def test_next_sequence_number(self):
        with self.app.app_context():
            sequence_number1 = self.service.get_next_sequence_number("test_sequence_1")
            sequence_number2 = self.service.get_next_sequence_number("test_sequence_1")
            self.assertEqual(sequence_number1 + 1, sequence_number2)

    def test_rotate_sequence_number(self):
        with self.app.app_context():

            last_sequence_number = None
            for i in range(self.max_seq_number):
                last_sequence_number = self.service.get_next_sequence_number(
                    "test_sequence_1", max_seq_number=self.max_seq_number, min_seq_number=self.min_seq_number
                )
            self.assertEqual(last_sequence_number, self.max_seq_number)

            for i in range(0, 2):
                last_sequence_number = self.service.get_next_sequence_number(
                    "test_sequence_1", max_seq_number=self.max_seq_number, min_seq_number=self.min_seq_number
                )
                self.assertEqual(last_sequence_number, self.min_seq_number + i, "failed for i={}".format(i))



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : GyanP
# Creation: 2020-03-09 12:19

from bson import ObjectId
from bson.errors import InvalidId
from superdesk import get_resource_service
from superdesk.vocabularies import is_related_content
from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "archive"

    def forwards(self, mongodb_collection, mongodb_database):
        for resource in ("archive", "published"):
            service = get_resource_service(resource)
            for item in mongodb_database[resource].find({"associations": {"$exists": "true", "$nin": [{}, None]}}):
                update = False
                associations = {}
                try:
                    for key, val in item["associations"].items():
                        associations[key] = val
                        if val and is_related_content(key) and val.get("order", None) is None:
                            update = True
                            order = int(key.split("--")[1])
                            associations[key]["order"] = order
                except AttributeError:
                    pass
                if update:
                    try:
                        _id = ObjectId(item["_id"])
                    except InvalidId:
                        _id = item["_id"]
                    service.system_update(_id, {"associations": associations}, item)

    def backwards(self, mongodb_collection, mongodb_database):
        raise NotImplementedError()



from .mimetype import Mimetype  # noqa
from .preferences import Preferences  # noqa
from .graphic import Graphic  # noqa
from .styles import Styles  # noqa
from .tags import Tags  # noqa
from .spread import Spread  # noqa
from .designmap import Designmap  # noqa
from .stories import Story, StoryTable, StoryList  # noqa
from .converter import Converter  # noqa



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock

from superdesk.tests import TestCase
from content_api.publish.service import PublishService
from apps.publish.enqueue.enqueue_service import EnqueueService
from apps.packages.package_service import PackageService
from apps.archive.archive import ArchiveService


def _fake_extend_subscriber_items(self, subscriber_items, subscribers, package_item, package_item_id, subscriber_codes):
    subscriber_items.clear()
    subscriber_items.update(
        {
            "8": {
                "subscriber": {
                    "_id": "toto",
                    "is_targetable": True,
                    "products": [],
                    "api_products": ["570e04e23c5e9f89fe95366e"],
                    "name": "content_api",
                    "subscriber_type": "all",
                    "destinations": [],
                    "is_active": True,
                    "sequence_num_settings": {"min": 1, "max": 999},
                    "api_enabled": True,
                },
                "items": {"8": "8"},
                "codes": [],
            }
        }
    )


class EnqueueServiceTest(TestCase):
    queue_items = [
        {
            "_id": 1,
            "destination": {"delivery_type": "ftp", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub1",
            "state": "pending",
            "associated_items": ["123"],
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "1",
            "item_version": 1,
            "publishing_action": "published",
        },
        {
            "_id": 2,
            "destination": {"delivery_type": "ftp", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub1",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "1",
            "item_version": 1,
            "publishing_action": "corrected",
            "associated_items": ["123"],
        },
        {
            "_id": 3,
            "destination": {"delivery_type": "ftp", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub2",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "2",
            "item_version": 1,
            "publishing_action": "published",
            "associated_items": ["123"],
        },
        {
            "_id": 4,
            "destination": {"delivery_type": "content_api", "format": "ninjs", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub2",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "2",
            "item_version": 2,
            "publishing_action": "corrected",
            "associated_items": ["456"],
        },
        {
            "_id": 5,
            "destination": {"delivery_type": "ftp", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub3",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "3",
            "item_version": 1,
            "publishing_action": "published",
            "associated_items": ["786"],
        },
        {
            "_id": 6,
            "destination": {"delivery_type": "content_api", "format": "ninjs", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub3",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "3",
            "item_version": 2,
            "publishing_action": "corrected",
        },
        {
            "_id": 7,
            "destination": {"delivery_type": "ftp", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub4",
            "state": "pending",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "2",
            "item_version": 1,
            "publishing_action": "published",
            "associated_items": ["123"],
        },
        {
            "_id": 8,
            "destination": {"delivery_type": "content_api", "format": "ninjs", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub4",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "2",
            "item_version": 2,
            "publishing_action": "corrected",
            "associated_items": ["456"],
        },
        {
            "_id": 9,
            "destination": {"delivery_type": "content_api", "format": "ninjs", "config": {}, "name": "destination1"},
            "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
            "subscriber_id": "sub5",
            "state": "success",
            "_created": "2015-04-17T13:15:20.000Z",
            "_updated": "2015-04-20T05:04:25.000Z",
            "item_id": "5",
            "item_version": 1,
            "publishing_action": "published",
        },
    ]
    content_api_package = {
        "_id": "10",
        "destination": {"delivery_type": "content_api", "format": "ninjs", "config": {}, "name": "destination1"},
        "_etag": "f28b9af64f169072fb171ec7f316fc03d5826d6b",
        "version": "2",
        "type": "composite",
        "headline": "test headline",
        "urgency": 3,
        "pubstatus": "usable",
        "slugline": "test slugline",
        "source": "foo",
        "associations": {"main-0": {"guid": "toto", "type": "text"}, "main-1": {"guid": "titi", "type": "text"}},
        "priority": 6,
        "subject": [{"code": "03003000", "name": "famine"}],
        "service": [{"code": "f", "name": "FIXME"}],
        "copyrightholder": "",
        "copyrightnotice": "",
        "usageterms": "",
        "state": "published",
        "genre": [{"name": "Article (news)", "code": "Article"}],
    }

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert("publish_queue", self.queue_items)

    def test_previously_sent_item_association_for_one_subscriber(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = service._get_subscribers_for_previously_sent_items(
            {"item_id": "1"}
        )
        self.assertEqual(len(associated_items.keys()), 1)
        self.assertIn("sub1", list(associated_items.keys()))
        self.assertIn("123", associated_items["sub1"])

    def test_previously_sent_item_association_for_multiple_subscribers(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = service._get_subscribers_for_previously_sent_items(
            {"item_id": "2"}
        )
        self.assertEqual(len(associated_items.keys()), 2)
        self.assertIn("sub2", associated_items)
        self.assertIn("sub4", associated_items)
        self.assertIn("123", associated_items["sub2"])
        self.assertIn("456", associated_items["sub2"])
        self.assertIn("123", associated_items["sub4"])
        self.assertIn("456", associated_items["sub4"])

    def test_previously_sent_item_association_for_removed_associations(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = service._get_subscribers_for_previously_sent_items(
            {"item_id": "3"}
        )
        self.assertEqual(len(associated_items.keys()), 1)
        self.assertIn("sub3", list(associated_items.keys()))
        self.assertIn("786", associated_items["sub3"])

    def test_previously_sent_item_association_for_no_associations(self):
        service = EnqueueService()
        subscribers, subscriber_codes, associated_items = service._get_subscribers_for_previously_sent_items(
            {"item_id": "5"}
        )
        self.assertEqual(len(associated_items.keys()), 0)

    @mock.patch.object(PackageService, "get_residrefs", lambda self, package: ["8", "9"])
    @mock.patch.object(ArchiveService, "find_one", lambda self, req, **lookup: EnqueueServiceTest.content_api_package)
    @mock.patch.object(EnqueueService, "_extend_subscriber_items", _fake_extend_subscriber_items)
    @mock.patch.object(EnqueueService, "queue_transmission", lambda *a, **kw: ([], True))
    @mock.patch.object(PublishService, "publish")
    def test_content_api_package_publishing(self, content_api_publish):
        service = EnqueueService()
        service.enqueue_item(self.content_api_package)
        # Mock.assert_called_once is only available in Python 3.6
        # so we emulate it by counting the number of calls
        assert content_api_publish.call_count == 1



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from lxml import etree  # noqa
from lxml.etree import ParseError  # noqa
from lxml import html
from superdesk import config


# from https://developer.mozilla.org/en-US/docs/Web/HTML/Block-level_elements
BLOCK_ELEMENTS = (
    "address",
    "article",
    "aside",
    "blockquote",
    "canvas",
    "dd",
    "div",
    "dl",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "header",
    "hgroup",
    "hr",
    "li",
    "main",
    "nav",
    "noscript",
    "ol",
    "output",
    "p",
    "pre",
    "section",
    "table",
    "tfoot",
    "ul",
    "video",
)

# from https://www.w3.org/TR/html/syntax.html#void-elements
VOID_ELEMENTS = (
    "area",
    "base",
    "br",
    "col",
    "embed",
    "hr",
    "img",
    "input",
    "keygen",
    "link",
    "menuitem",
    "meta",
    "param",
    "source",
    "track",
    "wbr",
)


def fix_html_void_elements(element):
    """Use self-closing elements for HTML void elements, and start/end pairs otherwise

    :param element: Element to fix
    :type element: lxml.etree.Element
    :return: fixed Element
    """
    # we want self closing for HTML void elemends and start/end tags otherwise
    # so we set element.text to None for void ones, and empty string otherwise
    for e in element.xpath("//*[not(node())]"):
        e.text = None if e.tag in VOID_ELEMENTS else ""
    return element


def parse_html(html, content="xml", lf_on_block=False, space_on_elements=False, space=" "):
    """Parse element and return etreeElement

    <div> element is added around the HTML
    recovery is used in case of bad markup
    :param str html: HTML markup
    :param str content: use 'xml' for XHTML or non html XML, and 'html' for HTML or if you are unsure
    :param bool lf_on_block: if True, add a line feed on block elements' tail
    :param bool space_on_elements: if True, add a space on each element's tail
        mainly used to count words with non HTML markup
    :param str space: space string which is used when `space_on_elements` is enabled
    :return etree.Element: parsed element
    """
    if not isinstance(html, str):
        raise ValueError("a string is expected")
    if not html:
        return etree.Element("div")

    if content == "xml":
        # to preserve 'carriage return' otherwise it gets stripped.
        html = html.replace("\r", "&#13;")
        parser = etree.XMLParser(recover=True, remove_blank_text=True)
        root = etree.fromstring("<div>" + html + "</div>", parser)
    elif content == "html":
        parser = etree.HTMLParser(recover=True, remove_blank_text=True)
        root = etree.fromstring(html, parser)
        if root is None:
            root = etree.Element("div")
        else:
            div = etree.Element("div")
            # we unwrap elements in <head> and <body>
            # <script> can be used in embed, and the parser will move them to <head>
            # so we need both <head> and <body>
            for elt in root:
                div.extend(elt)
            root = div
    else:
        raise ValueError("invalid content: {}".format(content))
    if lf_on_block:
        for elem in root.iterfind(".//"):
            # append \n to the tail
            if elem.tag in BLOCK_ELEMENTS:
                elem.tail = (elem.tail or "") + "\n"
            # prepend \n to the tail
            elif elem.tag in ("br",):
                elem.tail = "\n" + (elem.tail or "")
    if space_on_elements:
        for elem in root.iterfind(".//"):
            elem.tail = (elem.tail or "") + space
    return root


def to_string(elem, encoding="unicode", method="xml", remove_root_div=True, pretty_print=False):
    """Convert Element to string

    :param etree.Element elem: element to convert
    :param str encoding: encoding to use (same as for etree.tostring)
    :param str method: method to use (same as for etree.tostring)
    :param bool remove_root_dir: if True remove surrounding <div> which is added by parse_html
    :return str: converted element
    """
    string = etree.tostring(elem, encoding=encoding, method=method, pretty_print=pretty_print)
    if remove_root_div:
        if encoding == "unicode":
            div_start = "<div>"
            div_end = "</div>"
        else:
            div_start = b"<div>"
            div_end = b"</div>"
        if string.startswith(div_start) and string.endswith(div_end):
            return string[len(div_start) : -len(div_end)]
    return string


def clean_html(elem):
    """Clean HTML element by removing unknown or unsafe elements/attributes

    use config.HTML_TAGS_WHITELIST as list of known tags (i.e. tags understood by client)
    :param etree._Element elem: element to clean (will be converted to HtmlElement if it is not already one
    :return html.HtmlElement: cleaned element
    """
    if not isinstance(elem, html.HtmlElement):
        elem = html.fromstring(etree.tostring(elem))
    safe_attrs = set(html.defs.safe_attrs)
    safe_attrs.remove("class")
    cleaner = html.clean.Cleaner(
        allow_tags=config.HTML_TAGS_WHITELIST, remove_unknown_tags=False, safe_attrs=safe_attrs
    )
    return cleaner.clean_html(elem)


def clean_html_str(html_str):
    """Clean HTML like clean_html but using a string as input

    :param str html_str: raw HTML to clean
    :return str: cleaned HTML
    """
    html_elt = parse_html(html_str, "html")
    html_elt = clean_html(html_elt)
    return to_string(html_elt, method="html")



# This file is part of Superdesk.
#
# Copyright 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import bcrypt
from bson import ObjectId
from bson.errors import InvalidId
from authlib.oauth2.rfc6749 import ClientMixin
import superdesk
from .scopes import allowed_scopes

logger = logging.getLogger(__name__)
# client_id to OAuth2Client instance map


class OAuth2Client(ClientMixin):
    def __init__(self, data):
        self._id = data["_id"]
        self.pwd_hash = data["password"]
        scope = data["scope"]
        if not allowed_scopes.issuperset(scope):
            invalid_scopes = ", ".join(set(scope) - allowed_scopes)
            msg = (
                "Invalid scopes: those scope values are not allowed, "
                'please check "AUTH_SERVER_CLIENTS" in settings: {invalid_scopes}'.format(invalid_scopes=invalid_scopes)
            )
            raise ValueError(msg)
        self.scope = scope

    @property
    def client_id(self):
        return str(self._id)

    def check_token_endpoint_auth_method(self, method):
        return method == "client_secret_basic"

    def check_client_secret(self, client_secret):
        return bcrypt.checkpw(client_secret.encode(), self.pwd_hash.encode())

    def check_grant_type(self, grant_type):
        return grant_type == "client_credentials"

    def get_allowed_scope(self, scope):
        return ""


def query_client(client_id):
    clients_service = superdesk.get_resource_service("auth_server_clients")
    try:
        client_data = clients_service.find_one(req=None, _id=ObjectId(client_id))
    except InvalidId as e:
        logger.error("Invalid 'client_id' was provided. Exception: {}".format(e))
        return None

    if client_data is None:
        return None
    return OAuth2Client(client_data)


def save_token(token, request):
    # we don't save token as JWT signature is enough to check it
    pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import json

from superdesk.tests import TestCase
from superdesk import get_resource_service

from apps.prepopulate.app_populate import AppPopulateCommand


class AppPopulateValidatorsTest(TestCase):
    def setUp(self):
        self.filename = os.path.join(os.path.abspath(os.path.dirname(__file__)), "validators.json")
        self.json_data = [{"_id": "publish", "schema": {"headline": {"type": "string"}}}]

        with open(self.filename, "w+") as file:
            json.dump(self.json_data, file)

    def test_populate_validators(self):
        cmd = AppPopulateCommand()
        with self.app.test_request_context(self.app.config.get("URL_PREFIX")):
            cmd.run(self.filename)
            service = get_resource_service("validators")

            for item in self.json_data:
                data = service.find_one(_id=item["_id"], req=None)
                self.assertEqual(data["_id"], item["_id"])
                self.assertDictEqual(data["schema"], item["schema"])

    def tearDown(self):
        os.remove(self.filename)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import json

from eve.utils import ParsedRequest

from apps.content_filters.content_filter.content_filter_service import ContentFilterService
from apps.prepopulate.app_populate import AppPopulateCommand
from superdesk import get_backend, get_resource_service
from superdesk.errors import SuperdeskApiError
from superdesk.publish import SubscribersService
from superdesk.tests import TestCase


class ContentFilterTests(TestCase):
    def setUp(self):
        self.req = ParsedRequest()
        with self.app.test_request_context(self.app.config.get("URL_PREFIX")):
            self.f = ContentFilterService(datasource="content_filters", backend=get_backend())
            self.s = SubscribersService(datasource="subscribers", backend=get_backend())

            self.articles = [
                {"_id": "1", "urgency": 1, "headline": "story", "state": "fetched"},
                {"_id": "2", "headline": "prtorque", "state": "fetched"},
                {"_id": "3", "urgency": 3, "headline": "creator", "state": "fetched"},
                {"_id": "4", "urgency": 4, "state": "fetched"},
                {"_id": "5", "urgency": 2, "state": "fetched"},
                {"_id": "6", "state": "fetched"},
                {"_id": "7", "subject": [{"scheme": "my_vocabulary", "qcode": "MV:01"}]},
                {"_id": "8", "extra": {"custom_text": "my text"}},
            ]
            self.app.data.insert("archive", self.articles)

            self.app.data.insert(
                "vocabularies",
                [
                    {
                        "_id": "my_vocabulary",
                        "display_name": "My Vocabulary",
                        "type": "manageable",
                        "field_type": None,
                        "schema": {"name": {}, "qcode": {}, "parent": {}},
                        "items": [{"name": "option 1", "qcode": "MV:01", "is_active": True}],
                    },
                    {"_id": "custom_text", "display_name": "Custom Text", "type": "manageable", "field_type": "text"},
                ],
            )

            self.app.data.insert(
                "filter_conditions",
                [{"_id": 1, "field": "headline", "operator": "like", "value": "tor", "name": "test-1"}],
            )
            self.app.data.insert(
                "filter_conditions", [{"_id": 2, "field": "urgency", "operator": "in", "value": "2", "name": "test-2"}]
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 3, "field": "headline", "operator": "endswith", "value": "tor", "name": "test-3"}],
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 4, "field": "urgency", "operator": "in", "value": "2,3,4", "name": "test-4"}],
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 5, "field": "headline", "operator": "startswith", "value": "sto", "name": "test-5"}],
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 6, "field": "my_vocabulary", "operator": "in", "value": "MV:01", "name": "test-6"}],
            )
            self.app.data.insert(
                "filter_conditions",
                [{"_id": 7, "field": "custom_text", "operator": "eq", "value": "my text", "name": "test-7"}],
            )

            self.app.data.insert(
                "content_filters", [{"_id": 1, "content_filter": [{"expression": {"fc": [1]}}], "name": "soccer-only"}]
            )

            self.app.data.insert(
                "content_filters",
                [{"_id": 2, "content_filter": [{"expression": {"fc": [4, 3]}}], "name": "soccer-only2"}],
            )

            self.app.data.insert(
                "content_filters",
                [{"_id": 3, "content_filter": [{"expression": {"pf": [1], "fc": [2]}}], "name": "soccer-only3"}],
            )

            self.app.data.insert(
                "content_filters",
                [
                    {
                        "_id": 4,
                        "content_filter": [{"expression": {"fc": [3]}}, {"expression": {"fc": [5]}}],
                        "name": "soccer-only4",
                    }
                ],
            )

            self.app.data.insert(
                "content_filters",
                [{"_id": 5, "content_filter": [{"expression": {"fc": [6]}}], "name": "my-vocabulary"}],
            )

            self.app.data.insert(
                "content_filters", [{"_id": 6, "content_filter": [{"expression": {"fc": [7]}}], "name": "custom-text"}]
            )

            self.app.data.insert(
                "products", [{"_id": 1, "content_filter": {"filter_id": 3, "filter_type": "blocking"}, "name": "p-1"}]
            )

            self.app.data.insert(
                "products", [{"_id": 2, "content_filter": {"filter_id": 1, "filter_type": "blocking"}, "name": "p-2"}]
            )

            self.app.data.insert("subscribers", [{"_id": 1, "products": [1], "name": "sub1"}])
            self.app.data.insert("subscribers", [{"_id": 2, "products": [2], "name": "sub2"}])

            self.app.data.insert(
                "routing_schemes",
                [
                    {
                        "_id": 1,
                        "name": "routing_scheme_1",
                        "rules": [
                            {
                                "filter": 4,
                                "name": "routing_rule_4",
                                "schedule": {
                                    "day_of_week": ["MON"],
                                    "hour_of_day_from": "0000",
                                    "hour_of_day_to": "2355",
                                },
                                "actions": {"fetch": [], "publish": [], "exit": False},
                            }
                        ],
                    }
                ],
            )


class RetrievingDataTests(ContentFilterTests):
    def test_build_mongo_query_using_like_filter_single_fc(self):
        doc = {"content_filter": [{"expression": {"fc": [1]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(3, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("3" in doc_ids)

    def test_build_mongo_query_using_like_filter_single_pf(self):
        doc = {"content_filter": [{"expression": {"pf": [1]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(3, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("3" in doc_ids)

    def test_build_mongo_query_using_like_filter_multi_filter_condition(self):
        doc = {"content_filter": [{"expression": {"fc": [1]}}, {"expression": {"fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(4, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("5" in doc_ids)

    def test_build_mongo_query_using_like_filter_multi_pf(self):
        doc = {"content_filter": [{"expression": {"pf": [1]}}, {"expression": {"fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(4, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("5" in doc_ids)

    def test_build_mongo_query_using_like_filter_multi_filter_condition2(self):
        doc = {"content_filter": [{"expression": {"fc": [3, 4]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_mongo_query_using_like_filter_multi_pf2(self):
        doc = {"content_filter": [{"expression": {"pf": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_mongo_query_using_like_filter_multi_condition3(self):
        doc = {"content_filter": [{"expression": {"fc": [3, 4]}}, {"expression": {"fc": [1, 2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_mongo_query_using_like_filter_multi_pf3(self):
        doc = {"content_filter": [{"expression": {"pf": [2]}}, {"expression": {"pf": [1], "fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = self.f.build_mongo_query(doc)
            docs = get_resource_service("archive").get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_elastic_query_using_like_filter_single_filter_condition(self):
        doc = {"content_filter": [{"expression": {"fc": [1]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(3, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("3" in doc_ids)

    def test_build_elastic_query_using_like_filter_single_content_filter(self):
        doc = {"content_filter": [{"expression": {"pf": [1]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(3, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("3" in doc_ids)

    def test_build_elastic_query_using_like_filter_multi_filter_condition(self):
        doc = {"content_filter": [{"expression": {"fc": [1]}}, {"expression": {"fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(4, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("3" in doc_ids)
            self.assertTrue("5" in doc_ids)

    def test_build_mongo_query_using_like_filter_multi_content_filter(self):
        doc = {"content_filter": [{"expression": {"pf": [1]}}, {"expression": {"fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(4, docs.count())
            self.assertTrue("1" in doc_ids)
            self.assertTrue("2" in doc_ids)
            self.assertTrue("3" in doc_ids)
            self.assertTrue("5" in doc_ids)

    def test_build_elastic_query_using_like_filter_multi_filter_condition2(self):
        doc = {"content_filter": [{"expression": {"fc": [3, 4]}}, {"expression": {"fc": [1, 2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_elastic_query_using_like_filter_multi_content_filter2(self):
        doc = {
            "content_filter": [{"expression": {"fc": [4, 3]}}, {"expression": {"pf": [1], "fc": [2]}}],
            "name": "pf-1",
        }
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_elastic_query_using_like_filter_multi_content_filter3(self):
        doc = {"content_filter": [{"expression": {"pf": [2]}}, {"expression": {"pf": [1], "fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_elastic_query_using_like_filter_multi_content_filter4(self):
        doc = {"content_filter": [{"expression": {"pf": [2]}}, {"expression": {"pf": [3]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)

    def test_build_elastic_query_using_like_filter_multi_content_filter5(self):
        doc = {"content_filter": [{"expression": {"pf": [4], "fc": [4]}}], "name": "pf-1"}
        with self.app.app_context():
            query = {"query": {"filtered": {"query": self.f._get_elastic_query(doc)}}}
            self.req.args = {"source": json.dumps(query)}
            docs = get_resource_service("archive").get(req=self.req, lookup=None)
            doc_ids = [d["_id"] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue("3" in doc_ids)


class FilteringDataTests(ContentFilterTests):
    def test_does_match_returns_true_for_nonexisting_filter(self):
        for article in self.articles:
            self.assertTrue(self.f.does_match(None, article))

    def test_does_match_custom_vocabularies(self):
        doc1 = {"content_filter": [{"expression": {"fc": [6]}}], "name": "mv-1"}
        doc2 = {"content_filter": [{"expression": {"fc": [7]}}], "name": "ct-1"}
        with self.app.app_context():
            self.assertTrue(self.f.does_match(doc1, self.articles[6]))
            self.assertTrue(self.f.does_match(doc2, self.articles[7]))

    def test_does_match_using_like_filter_single_fc(self):
        doc = {"content_filter": [{"expression": {"fc": [1]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertTrue(self.f.does_match(doc, self.articles[0]))
            self.assertTrue(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertFalse(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_does_match_using_like_filter_single_pf(self):
        doc = {"content_filter": [{"expression": {"pf": [1]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertTrue(self.f.does_match(doc, self.articles[0]))
            self.assertTrue(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertFalse(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_does_match_using_like_filter_multi_fc(self):
        doc = {"content_filter": [{"expression": {"fc": [1]}}, {"expression": {"fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertTrue(self.f.does_match(doc, self.articles[0]))
            self.assertTrue(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertTrue(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_does_match_using_like_filter_multi_pf(self):
        doc = {"content_filter": [{"expression": {"pf": [1]}}, {"expression": {"fc": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertTrue(self.f.does_match(doc, self.articles[0]))
            self.assertTrue(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertTrue(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_does_match_using_like_filter_multi_fc2(self):
        doc = {"content_filter": [{"expression": {"fc": [3, 4]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertFalse(self.f.does_match(doc, self.articles[0]))
            self.assertFalse(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertFalse(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_does_match_using_like_filter_multi_pf2(self):
        doc = {"content_filter": [{"expression": {"pf": [2]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertFalse(self.f.does_match(doc, self.articles[0]))
            self.assertFalse(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertFalse(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_does_match_using_like_filter_multi_fc3(self):
        doc = {"content_filter": [{"expression": {"fc": [3, 4]}}, {"expression": {"fc": [1, 2]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertFalse(self.f.does_match(doc, self.articles[0]))
            self.assertFalse(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertFalse(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_does_match_using_like_filter_multi_pf3(self):
        doc = {"content_filter": [{"expression": {"pf": [4], "fc": [4]}}], "name": "pf-1"}
        with self.app.app_context():
            self.assertFalse(self.f.does_match(doc, self.articles[0]))
            self.assertFalse(self.f.does_match(doc, self.articles[1]))
            self.assertTrue(self.f.does_match(doc, self.articles[2]))
            self.assertFalse(self.f.does_match(doc, self.articles[3]))
            self.assertFalse(self.f.does_match(doc, self.articles[4]))
            self.assertFalse(self.f.does_match(doc, self.articles[5]))

    def test_if_pf_is_used(self):
        with self.app.app_context():
            self.assertTrue(self.f._get_content_filters_by_content_filter(1).count() == 1)
            self.assertTrue(self.f._get_content_filters_by_content_filter(4).count() == 0)

    def test_if_fc_is_used(self):
        with self.app.app_context():
            self.assertTrue(len(self.f.get_content_filters_by_filter_condition(1)) == 2)
            self.assertTrue(len(self.f.get_content_filters_by_filter_condition(3)) == 2)
            self.assertTrue(len(self.f.get_content_filters_by_filter_condition(2)) == 1)

    def test_get_subscribers_by_filter_condition(self):
        filter_condition1 = {"field": "urgency", "operator": "in", "value": "2"}
        filter_condition2 = {"field": "urgency", "operator": "in", "value": "1"}
        filter_condition3 = {"field": "headline", "operator": "like", "value": "tor"}
        filter_condition4 = {"field": "urgency", "operator": "nin", "value": "3"}

        with self.app.app_context():
            cmd = AppPopulateCommand()
            filename = os.path.join(
                os.path.abspath(os.path.dirname("apps/prepopulate/data_init/vocabularies.json")), "vocabularies.json"
            )
            cmd.run(filename)
            r1 = self.s._get_subscribers_by_filter_condition(filter_condition1)
            r2 = self.s._get_subscribers_by_filter_condition(filter_condition2)
            r3 = self.s._get_subscribers_by_filter_condition(filter_condition3)
            r4 = self.s._get_subscribers_by_filter_condition(filter_condition4)
            self.assertTrue(len(r1[0]["selected_subscribers"]) == 1)
            self.assertTrue(len(r2[0]["selected_subscribers"]) == 0)
            self.assertTrue(len(r3[0]["selected_subscribers"]) == 2)
            self.assertTrue(len(r4[0]["selected_subscribers"]) == 1)


class DeleteMethodTestCase(ContentFilterTests):
    """Tests for the delete() method."""

    def test_raises_error_if_filter_referenced_by_subscribers(self):
        with self.assertRaises(SuperdeskApiError) as ctx:
            self.f.delete({"_id": 1})

        self.assertEqual(ctx.exception.status_code, 400)  # bad request error

    def test_raises_error_if_filter_referenced_by_routing_rules(self):
        with self.assertRaises(SuperdeskApiError) as ctx:
            self.f.delete({"_id": 4})

        self.assertEqual(ctx.exception.status_code, 400)  # bad request error



from lxml import etree


from .base import BasePackageElement


class Graphic(BasePackageElement):
    """
    As Adobe's idml specification explains:
        ``
        The Graphic.xml file contains the links, colors, swatches, gradients, mixed inks, mixed ink groups, tints,
        and stroke styles contained in the document.
        ``
    """

    # Default indesign colours.
    # If defaults colors are not implemented, colors will not be available after idml was imported.
    COLORS = (
        {
            "Self": "Color/Black",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 0 0 100",
            "ColorOverride": "Specialblack",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "Black",
            "ColorEditable": "false",
            "ColorRemovable": "false",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch3",
        },
        {
            "Self": "Color/C=0 M=0 Y=100 K=0",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 0 100 0",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "C=0 M=0 Y=100 K=0",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch6",
        },
        {
            "Self": "Color/C=0 M=100 Y=0 K=0",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 100 0 0",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "C=0 M=100 Y=0 K=0",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch5",
        },
        {
            "Self": "Color/C=100 M=0 Y=0 K=0",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "100 0 0 0",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "C=100 M=0 Y=0 K=0",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch4",
        },
        {
            "Self": "Color/C=100 M=90 Y=10 K=0",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "100 90 10 0",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "C=100 M=90 Y=10 K=0",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch9",
        },
        {
            "Self": "Color/C=15 M=100 Y=100 K=0",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "15 100 100 0",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "C=15 M=100 Y=100 K=0",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch7",
        },
        {
            "Self": "Color/C=75 M=5 Y=100 K=0",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "75 5 100 0",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "C=75 M=5 Y=100 K=0",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch8",
        },
        {
            "Self": "Color/Cyan",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "100 0 0 0",
            "ColorOverride": "Hiddenreserved",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "Cyan",
            "ColorEditable": "false",
            "ColorRemovable": "false",
            "Visible": "false",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "n",
        },
        {
            "Self": "Color/Hyperlink",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "86 56.99999999999999 0 16",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "Hyperlink",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatcha",
        },
        {
            "Self": "Color/Magenta",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 100 0 0",
            "ColorOverride": "Hiddenreserved",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "Magenta",
            "ColorEditable": "false",
            "ColorRemovable": "false",
            "Visible": "false",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "n",
        },
        {
            "Self": "Color/Paper",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 0 0 0",
            "ColorOverride": "Specialpaper",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "Paper",
            "ColorEditable": "true",
            "ColorRemovable": "false",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch2",
        },
        {
            "Self": "Color/Registration",
            "Model": "Registration",
            "Space": "CMYK",
            "ColorValue": "100 100 100 100",
            "ColorOverride": "Specialregistration",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "Registration",
            "ColorEditable": "false",
            "ColorRemovable": "false",
            "Visible": "true",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "u18ColorGroupSwatch1",
        },
        {
            "Self": "Color/Yellow",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 0 100 0",
            "ColorOverride": "Hiddenreserved",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "Yellow",
            "ColorEditable": "false",
            "ColorRemovable": "false",
            "Visible": "false",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "n",
        },
        {
            "Self": "Color/u8d",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 0 0 100",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "$ID/",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "false",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "n",
        },
        {
            "Self": "Color/u8f",
            "Model": "Process",
            "Space": "CMYK",
            "ColorValue": "0 0 0 0",
            "ColorOverride": "Normal",
            "AlternateSpace": "NoAlternateColor",
            "AlternateColorValue": "",
            "Name": "$ID/",
            "ColorEditable": "true",
            "ColorRemovable": "true",
            "Visible": "false",
            "SwatchCreatorID": "7937",
            "SwatchColorGroupReference": "n",
        },
    )
    INKS = (
        {
            "Self": "Ink/$ID/Process Cyan",
            "Name": "$ID/Process Cyan",
            "Angle": "75",
            "ConvertToProcess": "false",
            "Frequency": "70",
            "NeutralDensity": "0.61",
            "PrintInk": "true",
            "TrapOrder": "1",
            "InkType": "Normal",
        },
        {
            "Self": "Ink/$ID/Process Magenta",
            "Name": "$ID/Process Magenta",
            "Angle": "15",
            "ConvertToProcess": "false",
            "Frequency": "70",
            "NeutralDensity": "0.76",
            "PrintInk": "true",
            "TrapOrder": "2",
            "InkType": "Normal",
        },
        {
            "Self": "Ink/$ID/Process Yellow",
            "Name": "$ID/Process Yellow",
            "Angle": "0",
            "ConvertToProcess": "false",
            "Frequency": "70",
            "NeutralDensity": "0.16",
            "PrintInk": "true",
            "TrapOrder": "3",
            "InkType": "Normal",
        },
        {
            "Self": "Ink/$ID/Process Black",
            "Name": "$ID/Process Black",
            "Angle": "45",
            "ConvertToProcess": "false",
            "Frequency": "70",
            "NeutralDensity": "1.7",
            "PrintInk": "true",
            "TrapOrder": "4",
            "InkType": "Normal",
        },
    )
    STROKESTYLES = (
        {"Self": "StrokeStyle/$ID/Triple_Stroke", "Name": "$ID/Triple_Stroke"},
        {"Self": "StrokeStyle/$ID/ThickThinThick", "Name": "$ID/ThickThinThick"},
        {"Self": "StrokeStyle/$ID/ThinThickThin", "Name": "$ID/ThinThickThin"},
        {"Self": "StrokeStyle/$ID/ThickThick", "Name": "$ID/ThickThick"},
        {"Self": "StrokeStyle/$ID/ThickThin", "Name": "$ID/ThickThin"},
        {"Self": "StrokeStyle/$ID/ThinThick", "Name": "$ID/ThinThick"},
        {"Self": "StrokeStyle/$ID/ThinThin", "Name": "$ID/ThinThin"},
        {"Self": "StrokeStyle/$ID/Japanese Dots", "Name": "$ID/Japanese Dots"},
        {"Self": "StrokeStyle/$ID/White Diamond", "Name": "$ID/White Diamond"},
        {"Self": "StrokeStyle/$ID/Left Slant Hash", "Name": "$ID/Left Slant Hash"},
        {"Self": "StrokeStyle/$ID/Right Slant Hash", "Name": "$ID/Right Slant Hash"},
        {"Self": "StrokeStyle/$ID/Straight Hash", "Name": "$ID/Straight Hash"},
        {"Self": "StrokeStyle/$ID/Wavy", "Name": "$ID/Wavy"},
        {"Self": "StrokeStyle/$ID/Canned Dotted", "Name": "$ID/Canned Dotted"},
        {"Self": "StrokeStyle/$ID/Canned Dashed 3x2", "Name": "$ID/Canned Dashed 3x2"},
        {"Self": "StrokeStyle/$ID/Canned Dashed 4x4", "Name": "$ID/Canned Dashed 4x4"},
        {"Self": "StrokeStyle/$ID/Dashed", "Name": "$ID/Dashed"},
        {"Self": "StrokeStyle/$ID/Solid", "Name": "$ID/Solid"},
    )

    @property
    def filename(self):
        """
        Filename inside IDML package.
        Used as a filename for a file inside zip container.
        :return str: filename
        """
        return "Resources/Graphic.xml"

    def _build_etree(self):
        self._etree = etree.Element(etree.QName(self.XMLNS_IDPKG, "Graphic"), nsmap={"idPkg": self.XMLNS_IDPKG})
        self._etree.set("DOMVersion", self.DOM_VERSION)
        self._add_colors()
        self._add_inks()
        self._add_pasted_smooth_shade()
        self._add_swatch()
        self._add_gradient()
        self._add_strokestyles()

    def _add_colors(self):
        """
        Add colors (<Color...>) defined `Graphic.COLORS`.
        As Adobe's idml specification explains:
            ``
            The <Color> element corresponds to a color in a document, including both named and unnamed colors.
            ``
        """
        for _attribs in self.COLORS:
            etree.SubElement(self._etree, "Color", attrib=_attribs)

    def _add_inks(self):
        """
        Add inks (<Ink...>) defined `Graphic.INKS`.
        As Adobe's idml specification explains:
            ``
            For process colors, Cyan, Magenta, Yellow and Black inks are mixed
            together to produce a range of colors.
            ``
        """
        for _attribs in self.INKS:
            etree.SubElement(self._etree, "Ink", attrib=_attribs)

    def _add_pasted_smooth_shade(self):
        """
        Add <PastedSmoothShade..>
        """
        pastedsmoothshade = etree.SubElement(
            self._etree,
            "PastedSmoothShade",
            attrib={
                "Self": "PastedSmoothShade/u90",
                "ContentsVersion": "0",
                "ContentsType": "ConstantShade",
                "SpotColorList": "",
                "ContentsEncoding": "Ascii64Encoding",
                "ContentsMatrix": "1 0 0 1 0 0",
                "Name": "$ID/",
                "ColorEditable": "true",
                "ColorRemovable": "true",
                "Visible": "false",
                "SwatchCreatorID": "7937",
                "SwatchColorGroupReference": "n",
            },
        )
        # Properties
        properties = etree.SubElement(pastedsmoothshade, "Properties")
        etree.SubElement(properties, "Contents").text = etree.CDATA("AAAAAUBv4AAAAAAAAAAAAAAAAAAAAAAAAAAAAA==")

    def _add_strokestyles(self):
        """
        Add stroke styles (<StrokeStyle...>) defined `Graphic.STROKESTYLES`.
        """
        for _attribs in self.STROKESTYLES:
            etree.SubElement(self._etree, "StrokeStyle", attrib=_attribs)

    def _add_swatch(self):
        """
        Add <Swatch..>.
        As Adobe's idml specification explains:
            ``
            In the InDesign user interface, colors, tints, gradients, and mixed inks are all swatches,
            and can be applied to the  ll or stroke of a page item or text.
            Swatches share similar attributes and are used in similar ways.
            Swatches are not the same as inks, but each swatch generally
            corresponds to one ink or to a speci c set of inks.
            A <Color> element, for example, might correspond to a single spot ink,
            or might be made up of percentages of process inks.
            ``
        """
        etree.SubElement(
            self._etree,
            "Swatch",
            attrib={
                "Self": "Swatch/None",
                "Name": "None",
                "ColorEditable": "false",
                "ColorRemovable": "false",
                "Visible": "true",
                "SwatchCreatorID": "7937",
                "SwatchColorGroupReference": "u18ColorGroupSwatch0",
            },
        )

    def _add_gradient(self):
        """
        Add <Gradient..>.
        As Adobe's idml specification explains:
           ``
           A gradient is a blend between two or more colors or between two tints of the same color.
           Gradients can include the “Paper” color, process colors, spot colors,
           or mixed inks using any color model or color space.
           A gradient is a blend between two or more colors or between two tints of the same color.
           Gradients can include the “Paper” color, process colors, spot colors,
           or mixed inks using any color model or color space.
           ``
        """
        gradient = etree.SubElement(
            self._etree,
            "Gradient",
            attrib={
                "Self": "Gradient/u8e",
                "Type": "Linear",
                "Name": "$ID/",
                "ColorEditable": "true",
                "ColorRemovable": "true",
                "Visible": "false",
                "SwatchCreatorID": "7937",
                "SwatchColorGroupReference": "n",
            },
        )
        etree.SubElement(
            gradient, "GradientStop", attrib={"Self": "u8eGradientStop0", "StopColor": "Color/u8f", "Location": "0"}
        )
        etree.SubElement(
            gradient,
            "GradientStop",
            attrib={"Self": "u8eGradientStop1", "StopColor": "Color/Black", "Location": "100", "Midpoint": "50"},
        )



import time

from superdesk.tests import TestCase
from superdesk.lock import lock, unlock, touch


class LockTestCase(TestCase):
    def test_lock(self):
        task = "test"

        self.assertTrue(lock(task, expire=2))
        self.assertFalse(lock(task))

        time.sleep(2)

        # lock again after expiry
        self.assertTrue(lock(task, expire=1))
        self.assertTrue(touch(task, expire=10))

        time.sleep(2)

        # can't lock after touch
        self.assertFalse(lock(task))

        unlock(task)

        # can't touch after unlock
        self.assertFalse(touch(task))

        # unlocking again is noop
        unlock(task)

        # locking after unlocking
        self.assertTrue(lock(task, expire=1))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import json
import csv
from pathlib import Path
from base64 import b64encode
from flask import current_app as app
import superdesk
from superdesk.utils import get_hash, is_hashed


logger = logging.getLogger(__name__)
USER_FIELDS_NAMES = {"username", "email", "password", "first_name", "last_name", "sign_off", "role"}


class CreateUserCommand(superdesk.Command):
    """Create a user with given username, password and email.

    If user with given username exists it's noop.

    Example:
    ::

        $ python manage.py users:create -u admin -p admin -e 'admin@example.com' --admin

    """

    option_list = (
        superdesk.Option("--username", "-u", dest="username", required=True),
        superdesk.Option("--password", "-p", dest="password", required=True),
        superdesk.Option("--email", "-e", dest="email", required=True),
        superdesk.Option("--admin", "-a", dest="admin", required=False, action="store_true"),
        superdesk.Option("--support", "-s", dest="support", required=False, action="store_true"),
    )

    def run(self, username, password, email, admin=False, support=False):

        # force type conversion to boolean
        user_type = "administrator" if admin else "user"

        userdata = {
            "username": username,
            "password": password,
            "email": email,
            "user_type": user_type,
            "is_active": admin,
            "is_support": support,
            "needs_activation": not admin,
        }

        with app.test_request_context("/users", method="POST"):
            if userdata.get("password", None) and not is_hashed(userdata.get("password")):
                userdata["password"] = get_hash(
                    userdata.get("password"), app.config.get("BCRYPT_GENSALT_WORK_FACTOR", 12)
                )

            user = superdesk.get_resource_service("users").find_one(username=userdata.get("username"), req=None)

            if user:
                logger.info("user already exists %s" % (userdata))
            else:
                logger.info("creating user %s" % (userdata))
                superdesk.get_resource_service("users").post([userdata])
                logger.info("user saved %s" % (userdata))

            return userdata


class ImportUsersCommand(superdesk.Command):
    """Imports users from JSON or CSV file.

    The file is a list of users, where the fields can be:

        - ``username`` (String), mandatory
          if there is already a user with given username, it will be skipped.
        - ``email`` (String), mandatory
          if there is already a user with given email, it will be skipped.
        - ``password`` (String)
        - ``first_name`` (String)
        - ``last_name`` (String)
        - ``sign_off`` (String)
        - ``role`` (String): name of an existing role to assign.
          The role must exist, otherwise the user will be skipped.
          Note that the role is case sensitive.

    For JSON, it's a list of object, each user must map field name to value.

    For CSV, fields names can be put in the first row of the CSV file.
    If the first row doesn't contain fields names, mapping between columns and fields
    must be done using ``-f`` (fields mut be specified in the same order as columns).

    File extension is used to detect if the file is JSON or CSV.

    Examples:
    ::

        $ python manage.py users:import path/to/users_list.json

        $ python manage.py users:import -f username -f email -f first_name -f last_name path/to/users_list.csv

    or, if the first row of your CSV file contains the fields names::

        $ python manage.py users:import path/to/users_list.csv

    """

    option_list = (
        superdesk.Option("--field", "-f", dest="fields", action="append"),
        superdesk.Option("--activation-email", "-a", dest="activation_email", required=False, action="store_true"),
        superdesk.Option("import_file"),
    )

    def run(self, fields, import_file, activation_email=False):
        import_path = Path(import_file)
        if import_path.suffix == ".csv":
            try:
                with import_path.open(newline="") as f:
                    reader = csv.DictReader(f, fieldnames=fields)
                    data = list(reader)
            except Exception as e:
                self.parser.error(
                    "Can't decode file at {path!r}, are you sure it's valid CSV? Error: {exc_msg}".format(
                        path=import_file, exc_msg=e
                    )
                )
        else:
            # we default to JSON even if the suffix is not ".json", as the parser will fail anyway if it's an other
            # format
            if fields is not None:
                self.parser.error("--field argument can only be used with CSV files")
            try:
                with open(import_file) as f:
                    data = json.load(f)
            except json.JSONDecodeError as e:
                self.parser.error(
                    "Can't decode file at {path!r}, are you sure it's valid JSON? Error: {exc_msg}".format(
                        path=import_file, exc_msg=e
                    )
                )

        if not isinstance(data, list):
            self.parser.error(
                "Invalid data file at {path!r}: import data must be a list of objects".format(path=import_file)
            )

        users_service = superdesk.get_resource_service("users")
        roles_service = superdesk.get_resource_service("roles")

        created_users = 0

        for user_data in data:
            if not isinstance(user_data, dict):
                self.parser.error(
                    "Invalid user data when importing {path!r}: user data must be an object, not {data_type}:\ndata: "
                    "{data!r}".format(path=import_file, data_type=type(user_data), data=user_data)
                )

            try:
                username = user_data["username"]
                email = user_data["email"]
                if not username.strip() or not email.strip():
                    raise KeyError
            except KeyError:
                logger.warning(
                    'Invalid user data when importing {path!r}: "username" and "email" are mandatory\ndata: '
                    "{data!r}".format(path=import_file, data=user_data)
                )
                continue

            diff_fields = set(user_data) - USER_FIELDS_NAMES
            if diff_fields:
                logger.warning(
                    "Invalid fields found ({invalid_fields}), they will be ignored. "
                    "Valid fields are: {valid_fields}".format(
                        invalid_fields=", ".join(sorted(diff_fields)), valid_fields=", ".join(sorted(USER_FIELDS_NAMES))
                    )
                )

            clean_data = {"needs_activation": activation_email}
            try:
                for field_name in USER_FIELDS_NAMES.intersection(user_data):
                    value = user_data[field_name]
                    if field_name == "role":
                        role_data = roles_service.find_one(req=None, name=value)
                        if role_data is None:
                            raise ValueError(
                                "The role {role!r} for user {username!r} is invalid, skipping user\ndata: {data!r}".format(
                                    role=value,
                                    username=username,
                                    data=data,
                                )
                            )
                        value = role_data["_id"]
                    clean_data[field_name] = value

                user_id = users_service.post([clean_data])[0]
            except Exception as e:
                logger.warning(
                    "Can't create user {username!r}: {reason}\n{data!r}".format(username=username, reason=e, data=data)
                )
                continue
            logger.info("user {username!r} created with id '{user_id}'".format(username=username, user_id=user_id))

            created_users += 1

        print(
            "{created_users}/{len_users} successfully created users".format(
                created_users=created_users, len_users=len(data)
            )
        )


class HashUserPasswordsCommand(superdesk.Command):
    """Hash all the user passwords which are not hashed yet.

    Example:
    ::

        $ python manage.py users:hash_passwords

    """

    def run(self):
        users = superdesk.get_resource_service("auth_users").get(req=None, lookup={})
        for user in users:
            pwd = user.get("password")
            if not is_hashed(pwd):
                updates = {}
                hashed = get_hash(user["password"], app.config.get("BCRYPT_GENSALT_WORK_FACTOR", 12))
                user_id = user.get("_id")
                updates["password"] = hashed
                superdesk.get_resource_service("users").patch(user_id, updates=updates)


class GetAuthTokenCommand(superdesk.Command):
    """Gets auth token.

    Generate an authorization token to be able to authenticate against the REST api without
    starting the client the copy the authorization header.

    Example:
    ::

        $ python manage.py users:get_auth_token --username=admin --password=123123

    """

    option_list = (
        superdesk.Option("--username", "-u", dest="username", required=True),
        superdesk.Option("--password", "-p", dest="password", required=True),
    )

    def run(self, username, password):
        credentials = {"username": username, "password": password}
        service = superdesk.get_resource_service("auth_db")
        id = str(service.post([credentials])[0])
        print("Session ID:", id)
        creds = service.find_one(req=None, _id=id)
        token = creds.get("token").encode("ascii")
        encoded_token = b"basic " + b64encode(token + b":")
        print("Generated token: ", encoded_token)
        return encoded_token


superdesk.command("users:create", CreateUserCommand())
superdesk.command("users:import", ImportUsersCommand())
superdesk.command("users:hash_passwords", HashUserPasswordsCommand())
superdesk.command("users:get_auth_token", GetAuthTokenCommand())



import unittest

from unittest.mock import patch
from superdesk.default_settings import celery_queue


class SettingsTestCase(unittest.TestCase):
    def test_celery_queue(self):
        with patch("superdesk.default_settings.os.environ.get", return_value="") as env:
            self.assertEqual("foo", celery_queue("foo"))
            env.assert_called_with("SUPERDESK_CELERY_PREFIX", "")
        with patch("superdesk.default_settings.os.environ.get", return_value="prefix"):
            self.assertEqual("prefixfoo", celery_queue("foo"))



from lxml import etree

from .story import Story


class StoryTable(Story):
    """
    Story which represents `table` html tag.
    """

    TABLE_DEFAULTS = {
        "HeaderRowCount": "0",
        "FooterRowCount": "0",
        "AppliedTableStyle": "TableStyle/$ID/[Basic Table]",
        "TableDirection": "LeftToRightDirection",
    }
    CELL_DEFAULTS = {"CellType": "TextTypeCell", "AppliedCellStyle": "CellStyle/$ID/[None]"}

    def __init__(self, self_id, element, inner_page_width, attributes=None, markup_tag=None):
        self._inner_page_width = inner_page_width
        super().__init__(self_id, element, attributes, markup_tag)

    def _add_story(self):
        # merge Story attributes
        story_attributes = self.merge_attributes(self.STORY_DEFAULTS, self._attributes.get("Story", {}))
        story_attributes.update({"Self": self.self_id})
        # Story
        story = etree.SubElement(self._etree, "Story", attrib=story_attributes)

        if self._markup_tag:
            # XMLElement to tag a story
            table_container = etree.SubElement(
                story,
                "XMLElement",
                attrib={
                    "Self": "{}_{}".format(self.self_id, self._markup_tag.lower()),
                    "XMLContent": self.self_id,
                    "MarkupTag": "XMLTag/{}".format(self._markup_tag),
                },
            )
        else:
            table_container = story

        # create Table and insert it into table_container
        table_container.insert(1, self._create_table())
        # StoryPreference
        etree.SubElement(
            story,
            "StoryPreference",
            attrib=self.merge_attributes(self.STORYPREFERENCE_DEFAULTS, self._attributes.get("StoryPreference", {})),
        )

        return story

    def _create_table(self):
        table_data = {}
        table_data["cells"] = self._element.xpath(".//td")
        table_data["rows_count"] = int(self._element.xpath("count(.//tr)"))
        table_data["columns_count"] = int(self._element.xpath("count(.//td)") / table_data["rows_count"])
        # Table
        table = etree.Element(
            "Table", attrib=self.merge_attributes(self.TABLE_DEFAULTS, self._attributes.get("Table", {}))
        )
        table.set("Self", "{}_table".format(self.self_id))
        table.set("BodyRowCount", str(table_data["rows_count"]))
        table.set("ColumnCount", str(table_data["columns_count"]))
        # Row(s)
        for i in range(table_data["rows_count"]):
            etree.SubElement(table, "Row", attrib={"Self": "{}_table_row{}".format(self.self_id, i), "Name": str(i)})
        # Column(s)
        column_width = self._inner_page_width / table_data["columns_count"]
        for i in range(table_data["columns_count"]):
            etree.SubElement(
                table,
                "Column",
                attrib={
                    "Self": "{}_table_column{}".format(self.self_id, i),
                    "Name": str(i),
                    "SingleColumnWidth": str(column_width),
                },
            )
        # Cells
        cell_counter = 0
        for r in range(table_data["rows_count"]):
            for c in range(table_data["columns_count"]):
                # Cell
                cell = etree.SubElement(
                    table,
                    "Cell",
                    attrib=self.merge_attributes(
                        self.CELL_DEFAULTS,
                        {
                            "Self": "{}_table_i{}".format(self.self_id, cell_counter),
                            "Name": "{cell}:{row}".format(cell=c, row=r),
                        },
                    ),
                )
                # CharacterStyleRange(s) + <Br />
                for p in table_data["cells"][cell_counter].xpath(".//p"):
                    if cell.find("CharacterStyleRange") is not None:
                        etree.SubElement(cell, "Br")
                    cell[:] += self._handle_inline_tags(p)
                cell_counter += 1

        return table

    @property
    def length(self):
        raise NotImplementedError

    @staticmethod
    def guess_height(story, inner_width):
        table_height = 0
        table_element = story._etree.xpath(".//Table")[0]
        column_count = int(table_element.get("ColumnCount"))
        row_count = int(table_element.get("BodyRowCount"))

        for row_number in range(row_count):
            highest_cell_len = None

            for column_number in range(column_count):
                try:
                    cell = story._etree.xpath('.//Cell[@Name="{}:{}"]'.format(column_number, row_number))[0]
                except IndexError:
                    continue
                else:
                    current_cell_len = 0

                    for content in cell.xpath(".//Content"):
                        current_cell_len += len(" ".join(etree.XPath(".//text()")(content)).strip())

                    if not highest_cell_len or current_cell_len > highest_cell_len:
                        highest_cell_len = current_cell_len

            row_height = highest_cell_len / (inner_width / column_count) * 90 + 10

            if row_height < 20:
                row_height = 20
            table_height += row_height

        return table_height



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from urllib.parse import urljoin
import responses
from flask import Flask
from superdesk.tests import TestCase
from superdesk.text_checkers import tools
from superdesk.text_checkers import spellcheckers
from superdesk.text_checkers.spellcheckers.base import registered_spellcheckers, SpellcheckerBase
from superdesk.text_checkers.spellcheckers.grammalecte import PATH_CHECK, PATH_SUGGEST, Grammalecte
from superdesk import get_resource_service
import os

spellcheckers.AUTO_IMPORT = False

TEST_URL = "http://localhost:8080"
os.environ["GRAMMALECTE_URL"] = TEST_URL


@responses.activate
def load_spellcheckers():
    """Load spellcheckers by mocking Grammalecte server, so it can be detected"""
    registered_spellcheckers.clear()
    app = Flask(__name__)
    check_url = urljoin(TEST_URL, PATH_CHECK)
    responses.add(
        responses.POST,
        check_url,
        json={
            "program": "grammalecte-fr",
            "version": "1.2",
        },
    )
    tools.import_services(app, spellcheckers.__name__, SpellcheckerBase)


class GrammalecteTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_spellcheckers()

    def test_list(self):
        """Check that Grammalecte is listed by spellcheckers_list service"""
        doc = {}
        spellcheckers_list = get_resource_service("spellcheckers_list")
        spellcheckers_list.on_fetched(doc)
        for checker in doc["spellcheckers"]:
            if (
                checker["name"] == Grammalecte.name
                and "label" in checker
                and set(checker["capacities"]) == set(Grammalecte.capacities)
                and checker["languages"] == ["fr"]
            ):
                return
        self.fail("Grammalecte not found")

    @responses.activate
    def test_checker(self):
        """Check that spellchecking is working"""
        doc = {
            "spellchecker": "grammalecte",
            "text": "Il nous reste à vérifié votre maquette.",
            "suggestions": False,
            "use_internal_dict": False,
        }
        spellchecker = get_resource_service("spellchecker")
        check_url = urljoin(TEST_URL, PATH_CHECK)
        responses.add(
            responses.POST,
            check_url,
            json={
                "program": "grammalecte-fr",
                "version": "1.2",
                "lang": "fr",
                "error": "",
                "data": [
                    {
                        "iParagraph": 1,
                        "lGrammarErrors": [
                            {
                                "nStart": 14,
                                "nEnd": 15,
                                "sLineId": "#5864",
                                "sRuleId": "conf_a_à_verbe__b13_a2_0",
                                "sType": "conf",
                                "aColor": [89, 49, 129],
                                "sMessage": "Confusion probable : “à” est une préposition. Pour le verbe “avoir”, écrivez “a”.",
                                "aSuggestions": ["a"],
                                "URL": "",
                            },
                            {
                                "nStart": 16,
                                "nEnd": 23,
                                "sLineId": "#5864",
                                "sRuleId": "conf_a_à_verbe__b13_a3_0",
                                "sType": "conf",
                                "aColor": [89, 49, 129],
                                "sMessage": "Le verbe devrait être à l’infinitif.",
                                "aSuggestions": ["vérifier"],
                                "URL": "",
                            },
                        ],
                        "lSpellingErrors": [],
                    }
                ],
            },
        )
        spellchecker.create([doc])

        expected = [
            {
                "startOffset": 14,
                "suggestions": [{"text": "a"}],
                "text": "à",
                "message": "Confusion probable : “à” est une préposition. " "Pour le verbe “avoir”, écrivez “a”.",
                "type": "grammar",
            },
            {
                "startOffset": 16,
                "suggestions": [{"text": "vérifier"}],
                "text": "vérifié",
                "message": "Le verbe devrait être à l’infinitif.",
                "type": "grammar",
            },
        ]

        self.assertEqual(doc["errors"], expected)

    @responses.activate
    def test_checker_paragraphs(self):
        """Check that spellchecking is working as expected with 2 paragraphs"""
        doc = {
            "spellchecker": "grammalecte",
            "text": "Il nous reste à vérifié votre maquette.\n\nIl nous reste à vérifié votre maquette.",
            "suggestions": False,
            "use_internal_dict": False,
        }
        spellchecker = get_resource_service("spellchecker")
        check_url = urljoin(TEST_URL, PATH_CHECK)
        responses.add(
            responses.POST,
            check_url,
            json={
                "program": "grammalecte-fr",
                "version": "1.1",
                "lang": "fr",
                "error": "",
                "data": [
                    {
                        "iParagraph": 1,
                        "lGrammarErrors": [
                            {
                                "URL": "",
                                "aColor": [89, 49, 129],
                                "aSuggestions": ["a"],
                                "nEnd": 15,
                                "nStart": 14,
                                "sLineId": "#5643",
                                "sMessage": "Confusion probable : “à” est une "
                                "préposition. Pour le verbe "
                                "“avoir”, écrivez “a”.",
                                "sRuleId": "conf_a_à_verbe__b13_a2_0",
                                "sType": "conf",
                            },
                            {
                                "URL": "",
                                "aColor": [89, 49, 129],
                                "aSuggestions": ["vérifier"],
                                "nEnd": 23,
                                "nStart": 16,
                                "sLineId": "#5643",
                                "sMessage": "Le verbe devrait être à " "l’infinitif.",
                                "sRuleId": "conf_a_à_verbe__b13_a3_0",
                                "sType": "conf",
                            },
                        ],
                        "lSpellingErrors": [],
                    },
                    {
                        "iParagraph": 3,
                        "lGrammarErrors": [
                            {
                                "URL": "",
                                "aColor": [89, 49, 129],
                                "aSuggestions": ["a"],
                                "nEnd": 15,
                                "nStart": 14,
                                "sLineId": "#5643",
                                "sMessage": "Confusion probable : “à” est une "
                                "préposition. Pour le verbe "
                                "“avoir”, écrivez “a”.",
                                "sRuleId": "conf_a_à_verbe__b13_a2_0",
                                "sType": "conf",
                            },
                            {
                                "URL": "",
                                "aColor": [89, 49, 129],
                                "aSuggestions": ["vérifier"],
                                "nEnd": 23,
                                "nStart": 16,
                                "sLineId": "#5643",
                                "sMessage": "Le verbe devrait être à " "l’infinitif.",
                                "sRuleId": "conf_a_à_verbe__b13_a3_0",
                                "sType": "conf",
                            },
                        ],
                        "lSpellingErrors": [],
                    },
                ],
            },
        )
        spellchecker.create([doc])

        expected = [
            {
                "message": "Confusion probable : “à” est une préposition. Pour le " "verbe “avoir”, écrivez “a”.",
                "startOffset": 14,
                "suggestions": [{"text": "a"}],
                "text": "à",
                "type": "grammar",
            },
            {
                "message": "Le verbe devrait être à l’infinitif.",
                "startOffset": 16,
                "suggestions": [{"text": "vérifier"}],
                "text": "vérifié",
                "type": "grammar",
            },
            {
                "message": "Confusion probable : “à” est une préposition. Pour le " "verbe “avoir”, écrivez “a”.",
                "startOffset": 55,
                "suggestions": [{"text": "a"}],
                "text": "à",
                "type": "grammar",
            },
            {
                "message": "Le verbe devrait être à l’infinitif.",
                "startOffset": 57,
                "suggestions": [{"text": "vérifier"}],
                "text": "vérifié",
                "type": "grammar",
            },
        ]

        self.assertEqual(doc["errors"], expected)

    @responses.activate
    def test_suggest(self):
        """Check that spelling suggestions are working"""

        doc = {
            "spellchecker": "grammalecte",
            "text": "fote",
            "suggestions": True,
        }
        spellchecker = get_resource_service("spellchecker")
        check_url = urljoin(TEST_URL, PATH_SUGGEST)
        responses.add(
            responses.POST,
            check_url,
            json={
                "suggestions": ["faute", "fauté", "féauté", "sotte", "rote", "roté", "note", "noté", "lote", "lotte"]
            },
        )
        spellchecker.create([doc])

        self.assertEqual(
            doc,
            {
                "spellchecker": "grammalecte",
                "suggestions": [
                    {"text": "faute"},
                    {"text": "fauté"},
                    {"text": "féauté"},
                    {"text": "sotte"},
                    {"text": "rote"},
                    {"text": "roté"},
                    {"text": "note"},
                    {"text": "noté"},
                    {"text": "lote"},
                    {"text": "lotte"},
                ],
                "text": "fote",
            },
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import pytz
import unittest
from datetime import datetime, timedelta
from superdesk.utc import (
    get_date,
    utcnow,
    get_expiry_date,
    local_to_utc,
    utc_to_local,
    get_timezone_offset,
    query_datetime,
)
from pytz import utc, timezone  # flake8: noqa
from nose.tools import assert_raises


class UTCTestCase(unittest.TestCase):
    def test_get_date(self):
        self.assertIsInstance(get_date("2012-12-12"), datetime)
        self.assertIsInstance(get_date(datetime.now()), datetime)
        self.assertIsNone(get_date(None))

    def test_utcnow(self):
        self.assertIsInstance(utcnow(), datetime)
        date1 = get_date(datetime.now(tz=utc))
        date2 = utcnow()
        self.assertEqual(date1.year, date2.year)
        self.assertEqual(date1.month, date2.month)
        self.assertEqual(date1.day, date2.day)
        self.assertEqual(date1.hour, date2.hour)
        self.assertEqual(date1.minute, date2.minute)
        self.assertEqual(date1.second, date2.second)

    def test_get_expiry_date(self):
        self.assertIsInstance(get_expiry_date(minutes=60), datetime)
        date1 = utcnow() + timedelta(minutes=60)
        date2 = get_expiry_date(minutes=60)
        self.assertEqual(date1.year, date2.year)
        self.assertEqual(date1.month, date2.month)
        self.assertEqual(date1.day, date2.day)
        self.assertEqual(date1.hour, date2.hour)
        self.assertEqual(date1.minute, date2.minute)
        self.assertEqual(date1.second, date2.second)

    def test_get_expiry_date_none(self):
        self.assertIsNone(get_expiry_date(0))
        self.assertIsNone(get_expiry_date(None))

    def test_get_expiry_date_with_offset(self):
        offset = utcnow() + timedelta(minutes=10)
        date1 = offset + timedelta(minutes=5)
        date2 = get_expiry_date(minutes=5, offset=offset)
        self.assertEqual(date1.year, date2.year)
        self.assertEqual(date1.month, date2.month)
        self.assertEqual(date1.day, date2.day)
        self.assertEqual(date1.hour, date2.hour)
        self.assertEqual(date1.minute, date2.minute)
        self.assertEqual(date1.second, date2.second)

    def test_get_expiry_date_bad_offset_raises_error(self):
        with assert_raises(TypeError) as error_context:
            offset = "01.02.2013 13:30"
            get_expiry_date(minutes=5, offset=offset)

    def test_get_expiry_date_overflow(self):
        self.assertIsNone(get_expiry_date(9999999999999))
        self.assertIsNone(get_expiry_date(9999999999999, utcnow()))

    def test_utc_to_local(self):
        # with day light saving on
        utc_dt = datetime(2015, 2, 8, 23, 0, 0, 0, pytz.UTC)
        local_dt = utc_to_local("Australia/Sydney", utc_dt)
        self.assertEqual(utc_dt.hour - local_dt.hour, 13)

        # without day light saving
        utc_dt = datetime(2015, 6, 8, 23, 0, 0, 0, pytz.UTC)
        local_dt = utc_to_local("Australia/Sydney", utc_dt)
        self.assertEqual(utc_dt.hour - local_dt.hour, 14)

    def test_local_to_utc(self):
        # with day light saving on
        local_tz = pytz.timezone("Australia/Sydney")
        local_dt = datetime(2015, 2, 8, 18, 0, 0, 0, local_tz)
        utc_dt = local_to_utc("Australia/Sydney", local_dt)
        self.assertEqual(local_dt.hour - utc_dt.hour, 11)

        # without day light saving
        local_dt = local_tz.normalize(datetime(2015, 6, 8, 18, 0, 0, 0).replace(tzinfo=local_tz))
        utc_dt = local_to_utc("Australia/Sydney", local_dt)
        self.assertEqual(local_dt.hour - utc_dt.hour, 10)

    def test_local_to_utc_europe(self):
        utc_dt = local_to_utc("Europe/Prague", datetime(2016, 4, 19, 15, 8, 0))
        self.assertEqual("2016-04-19T13:08:00+00:00", utc_dt.isoformat())

    def test_get_timezone_offset_sydney(self):
        utc_dt = datetime(2015, 2, 8, 23, 0, 0, 0, pytz.UTC)
        offset = get_timezone_offset("Australia/Sydney", utc_dt)
        self.assertEqual(offset, "+1100")

        utc_dt = datetime(2015, 9, 8, 23, 0, 0, 0, pytz.UTC)
        offset = get_timezone_offset("Australia/Sydney", utc_dt)
        self.assertEqual(offset, "+1000")

    def test_get_timezone_offset_prague(self):
        utc_dt = datetime(2015, 2, 8, 23, 0, 0, 0, pytz.UTC)
        offset = get_timezone_offset("Europe/Prague", utc_dt)
        self.assertEqual(offset, "+0100")

        utc_dt = datetime(2015, 9, 8, 23, 0, 0, 0, pytz.UTC)
        offset = get_timezone_offset("Europe/Prague", utc_dt)
        self.assertEqual(offset, "+0200")

    def test_get_timezone_offset_wrong_input(self):
        utc_dt = "test"
        offset = get_timezone_offset("Europe/Prague", utc_dt)
        self.assertEqual(offset, "+0000")

        utc_dt = datetime(2015, 9, 8, 23, 0, 0, 0, pytz.UTC)
        offset = get_timezone_offset("Europe/Sydney", utc_dt)
        self.assertEqual(offset, "+0000")


class UTCQueryTest(unittest.TestCase):
    def setUp(self):
        self.now = utcnow()
        self.past = self.now - timedelta(hours=1)
        self.future = self.now + timedelta(hours=1)

    def test_lte(self):
        self.assertTrue(query_datetime(self.past, {"$lte": self.now}))
        self.assertTrue(query_datetime(self.now, {"$lte": self.now}))
        self.assertFalse(query_datetime(self.future, {"$lte": self.now}))

    def test_lt(self):
        self.assertTrue(query_datetime(self.past, {"$lt": self.now}))
        self.assertFalse(query_datetime(self.now, {"$lt": self.now}))
        self.assertFalse(query_datetime(self.future, {"$lt": self.now}))

    def test_gte(self):
        self.assertFalse(query_datetime(self.past, {"$gte": self.now}))
        self.assertTrue(query_datetime(self.now, {"$gte": self.now}))
        self.assertTrue(query_datetime(self.future, {"$gte": self.now}))

    def test_gt(self):
        self.assertFalse(query_datetime(self.past, {"$gt": self.now}))
        self.assertFalse(query_datetime(self.now, {"$gt": self.now}))
        self.assertTrue(query_datetime(self.future, {"$gt": self.now}))

    def test_eq(self):
        self.assertFalse(query_datetime(self.past, {"$eq": self.now}))
        self.assertTrue(query_datetime(self.now, {"$eq": self.now}))
        self.assertFalse(query_datetime(self.future, {"$eq": self.now}))

    def test_neq(self):
        self.assertTrue(query_datetime(self.past, {"$ne": self.now}))
        self.assertFalse(query_datetime(self.now, {"$ne": self.now}))
        self.assertTrue(query_datetime(self.future, {"$ne": self.now}))

    def test_combination(self):
        self.assertTrue(query_datetime(self.now, {"$lt": self.future, "$gt": self.past}))
        self.assertFalse(query_datetime(self.past, {"$lt": self.future, "$gt": self.now}))



import unittest
from superdesk.filemeta import get_filemeta, set_filemeta


class FilemetaTestCase(unittest.TestCase):
    def test_get_set_filemeta(self):
        item = {}
        set_filemeta(item, {"foo": "bar"})
        self.assertEqual("bar", get_filemeta(item, "foo"))
        self.assertEqual({"foo": "bar"}, get_filemeta(item))

    def test_get_filemeta_obsolete(self):
        item = {"filemeta": {"x": 1}}
        self.assertEqual(1, get_filemeta(item, "x"))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json

from flask import current_app as app
from superdesk.emails import send_email
from superdesk.publish import register_transmitter
from superdesk.publish.publish_service import PublishService
from superdesk.errors import PublishEmailError
from superdesk.media.media_operations import get_watermark
from flask_mail import Attachment
from PIL import Image
import io
import logging

errors = [PublishEmailError.emailError().get_error_description()]

logger = logging.getLogger(__name__)


class EmailPublishService(PublishService):
    """Email Transmitter

    Works only with email formatter.

    :param recipients: email addresses separated by ``;``
    """

    NAME = "Email"

    def _transmit(self, queue_item, subscriber):
        config = queue_item.get("destination", {}).get("config", {})

        try:
            # detect if it's been formatted by the Email formatter, is so load the item
            try:
                item = json.loads(queue_item["formatted_item"])
                if "message_subject" not in item:
                    item = {}
            except Exception:
                item = {}

            admins = app.config["ADMINS"]
            recipients = [r.strip() for r in config.get("recipients", "").split(";") if r.strip()]
            bcc = [r.strip() for r in config.get("recipients_bcc", "").split(";") if r.strip()]
            if not recipients and not bcc:
                raise PublishEmailError.recipientNotFoundError(LookupError("recipient and bcc fields are empty!"))

            subject = item.get("message_subject", "Story: {}".format(queue_item["item_id"]))
            text_body = item.get("message_text", queue_item["formatted_item"])
            html_body = item.get("message_html", queue_item["formatted_item"])

            # Attach feature media if required
            attachments = []
            if config.get("attach_media") and item.get("renditions"):
                # The CID can be used to embbed the image in the html template email like <img src=CID:cid-value>
                cid = config.get("media_cid", "")
                # Get the rendition that has been nominated for attaching to the email
                rendition = config.get("media_rendition", "")
                media_item = item.get("renditions", {}).get(rendition)
                if media_item and rendition:
                    media = app.media.get(media_item["media"], resource="upload")
                    im = Image.open(media)
                    if config.get("watermark", False):
                        im = get_watermark(im)

                    binary = io.BytesIO()
                    im.save(binary, "jpeg", quality=80)

                    attachments.append(
                        Attachment(
                            filename=media.name,
                            content_type=media.content_type,
                            data=binary.getvalue(),
                            headers=[("Content-ID", cid)],
                        )
                    )

            # sending email synchronously
            send_email(
                subject=subject,
                sender=admins[0],
                recipients=recipients,
                text_body=text_body,
                html_body=html_body,
                bcc=bcc,
                attachments=attachments,
            )

        except Exception as ex:
            raise PublishEmailError.emailError(ex, queue_item.get("destination"))


register_transmitter("email", EmailPublishService(), errors)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from .ninjs_formatter import NINJSFormatter
import superdesk


class NewsroomNinjsFormatter(NINJSFormatter):
    def __init__(self):
        self.format_type = "newsroom ninjs"
        self.can_preview = False
        self.can_export = False
        self.internal_renditions = ["original", "viewImage", "baseImage"]

    def _format_products(self, article):
        """
        Return a list of API product id's that the article matches.

        :param article:
        :return:
        """
        result = superdesk.get_resource_service("product_tests").test_products(article, lookup=None)
        return [{"code": p["product_id"], "name": p.get("name")} for p in result if p.get("matched", False)]

    def _transform_to_ninjs(self, article, subscriber, recursive=True):
        ninjs = super()._transform_to_ninjs(article, subscriber, recursive)

        if article.get("ingest_id") and article.get("auto_publish"):
            ninjs["guid"] = article.get("ingest_id")
            ninjs["version"] = article.get("ingest_version")

        ninjs["products"] = self._format_products(article)

        if article.get("assignment_id"):
            assignment = superdesk.get_resource_service("assignments").find_one(req=None, _id=article["assignment_id"])
            if assignment is not None:
                if assignment.get("coverage_item"):
                    ninjs.setdefault("coverage_id", assignment["coverage_item"])
                if assignment.get("planning_item"):
                    ninjs.setdefault("planning_id", assignment["planning_item"])

        return ninjs



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from typing import List, Dict
import traceback
import requests
from superdesk.errors import IngestApiError, SuperdeskIngestError
from superdesk.io.feeding_services import FeedingService


class HTTPFeedingServiceBase(FeedingService):
    """
    Base class for feeding services using HTTP.

    This class contains helpers to make the creation of HTTP based feeding services
    easier.

    There are a couple of class attributes you can use:

    =======================  ===========
    Attribute                Explanation
    =======================  ===========
    HTTP_URL                 Main URL of your service, will be used by default in get_url
    HTTP_TIMEOUT             Timeout of requests in seconds
    HTTP_DEFAULT_PARAMETERS  Parameters used in every ``get`` requests.
                             Will be updated with params set in arguments
    HTTP_AUTH                Indicate if HTTP authentication is needed for your service.
                             If None, the authentication will be determined by the existence
                             of user and password. Will be overriden by auth_required config
                             if it exists.
    =======================  ===========

    In addition, you have some pre-filled fields:

    ===============  ===========
    Field            Explanation
    ===============  ===========
    AUTH_FIELDS      username and password fields
    AUTH_REQ_FIELDS  username and password fields + auth_required field to indicate
                     if they are needed
    ===============  ===========

    When ingest is updated, the provider is automatically saved to ``self.provider``.
    ``config`` property allows to access easily the user configuration.
    ``auth_info`` property returns a dictionary with ``username`` and ``password``

    ``get_url`` method do a HTTP Get request. url can be ommited in which case HTTP_URL will be used.
    Authentication parameters are set automatically, and errors are catched appropriately.
    Extra arguments are used directly in *requests* call.

    """

    ERRORS = [
        IngestApiError.apiTimeoutError().get_error_description(),
        IngestApiError.apiRequestError().get_error_description(),
        IngestApiError.apiGeneralError().get_error_description(),
        SuperdeskIngestError.notConfiguredError().get_error_description(),
    ]

    # override this parameter with the main URL to use
    HTTP_URL = None
    # timeout in seconds
    HTTP_TIMEOUT = 30
    # if some parameters are used in every request, put them here
    HTTP_DEFAULT_PARAMETERS = None
    # Set to True if authentication is mandatory, False if there is no authentication
    # and None to add authentication if user and password are defined.
    # If auth_required is defined in config fields, it will override this value.
    HTTP_AUTH = True

    # use this when auth is always required
    AUTH_FIELDS: List[Dict] = [
        {"id": "username", "type": "text", "label": "Username", "placeholder": "Username", "required": True},
        {"id": "password", "type": "password", "label": "Password", "placeholder": "Password", "required": True},
    ]

    # use this when auth depends of a "auth_required" flag (set by user)
    AUTH_REQ_FIELDS: List[Dict] = [
        {
            "id": "auth_required",
            "type": "boolean",
            "label": "Requires Authentication",
            "placeholder": "Requires Authentication",
            "required": False,
        },
        {
            "id": "username",
            "type": "text",
            "label": "Username",
            "placeholder": "Username",
            "required_expression": "{auth_required}",
            "show_expression": "{auth_required}",
        },
        {
            "id": "password",
            "type": "password",
            "label": "Password",
            "placeholder": "Password",
            "required_expression": "{auth_required}",
            "show_expression": "{auth_required}",
        },
    ]

    def __init__(self):
        super().__init__()
        self.token = None

    @property
    def auth_info(self):
        """Helper method to retrieve a dict with username and password when set"""
        username = self.config.get("username", "")
        password = self.config.get("password", "")
        if not username or not password:
            return None
        return {"username": username, "password": password}

    @property
    def config(self):
        return self.provider.setdefault("config", {})

    def validate_config(self):
        """
        Validate provider config according to `cls.fields`

        :param config: Ingest provider configuration
        :type config: dict
        :return:
        """
        # validate required config fields
        required_keys = [field["id"] for field in self.fields if field.get("required", False)]
        if not set(self.config.keys()).issuperset(required_keys):
            raise SuperdeskIngestError.notConfiguredError(
                Exception("{} are required.".format(", ".join(required_keys)))
            )

        # validate url
        url = self.config.get("url")
        if url and not url.strip().startswith("http"):
            raise SuperdeskIngestError.notConfiguredError(Exception("URL must be a valid HTTP link."))

    def get_url(self, url=None, **kwargs):
        """Do an HTTP Get on URL

        :param string url: url to use (None to use self.HTTP_URL)
        :param **kwargs: extra parameter for requests
        :return requests.Response: response
        """
        if not url:
            url = self.HTTP_URL
        config = self.config
        user = config.get("username")
        password = config.get("password")
        if user:
            user = user.strip()
        if password:
            password = password.strip()

        auth_required = config.get("auth_required", self.HTTP_AUTH)
        if auth_required is None:
            # auth_required may not be user in the feeding service
            # in this case with use authentification only if user
            # and password are set.
            auth_required = bool(user and password)

        if auth_required:
            if not user:
                raise SuperdeskIngestError.notConfiguredError("user is not configured")
            if not password:
                raise SuperdeskIngestError.notConfiguredError("password is not configured")
            kwargs.setdefault("auth", (user, password))

        params = kwargs.pop("params", {})
        if params or self.HTTP_DEFAULT_PARAMETERS:
            # if we have default parameters, we want them to be overriden
            # by conflicting params given in arguments
            if self.HTTP_DEFAULT_PARAMETERS:
                params.update(self.HTTP_DEFAULT_PARAMETERS)
            kwargs["params"] = params

        try:
            response = requests.get(url, timeout=self.HTTP_TIMEOUT, **kwargs)
        except requests.exceptions.Timeout as exception:
            raise IngestApiError.apiTimeoutError(exception, self.provider)
        except requests.exceptions.ConnectionError as exception:
            raise IngestApiError.apiConnectionError(exception, self.provider)
        except requests.exceptions.RequestException as exception:
            raise IngestApiError.apiRequestError(exception, self.provider)
        except Exception as exception:
            traceback.print_exc()
            raise IngestApiError.apiGeneralError(exception, self.provider)

        if not response.ok:
            exc = Exception(response.reason)
            if response.status_code in (401, 403):
                raise IngestApiError.apiAuthError(exc, self.provider)
            elif response.status_code == 404:
                raise IngestApiError.apiNotFoundError(exc, self.provider)
            else:
                raise IngestApiError.apiGeneralError(exc, self.provider)

        return response

    def update(self, provider, update):
        self.provider = provider
        self.validate_config()
        return super().update(provider, update)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from superdesk import get_backend
from apps.marked_desks.resource import MarkedForDesksResource
from apps.marked_desks.service import MarkedForDesksService


def init_app(app):
    endpoint_name = "marked_for_desks"
    service = MarkedForDesksService(endpoint_name, backend=get_backend())
    MarkedForDesksResource(endpoint_name, app=app, service=service)

    superdesk.privilege(
        name="mark_for_desks", label="Mark items for desks", description="User can mark items for other desks."
    )
    superdesk.privilege(
        name="mark_for_desks__non_members",
        label="Mark items for desks even if the user is not a member of the desk",
        description="User can mark items for other desks even if they are not a member of that desk",
    )



import superdesk


def init_app(app):
    app.client_config.update(
        {
            "tansa": {
                "base_url": app.config.get("TANSA_CLIENT_BASE_URL"),
                "app_id": app.config.get("TANSA_APP_ID"),
                "app_version": superdesk.__version__,
                "user_id": app.config.get("TANSA_USER_ID"),
                "profile_id": int(app.config["TANSA_PROFILE_ID"]) if app.config.get("TANSA_PROFILE_ID") else None,
                "license_key": app.config.get("TANSA_LICENSE_KEY"),
                "profiles": app.config.get("TANSA_PROFILES"),
            },
        }
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import unittest
import flask

from xml.etree import ElementTree
from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser
from superdesk.io.subjectcodes import init_app as init_subjects


class BaseNewMLTwoTestCase(unittest.TestCase):
    def setUp(self):
        app = flask.Flask(__name__)
        app.api_prefix = "/api"
        init_subjects(app)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            self.parser = NewsMLTwoFeedParser()
            self.xml = ElementTree.parse(f)
            with app.app_context():
                self.item = self.parser.parse(self.xml.getroot(), provider)


class ReutersTestCase(BaseNewMLTwoTestCase):
    filename = "tag:reuters.com,0000:newsml_L4N1FL0N0:1132689232"

    def test_content(self):
        self.assertEqual(self.item[0].get("headline"), "PRECIOUS-Gold rises as Trump policy fuels safe haven demand")
        self.assertEqual(self.item[0].get("guid"), "tag:reuters.com,0000:newsml_L4N1FL0N0:1132689232")
        self.assertEqual(self.item[0].get("uri"), "tag:reuters.com,0000:newsml_L4N1FL0N0")
        self.assertEqual(self.item[0].get("firstcreated").isoformat(), "2017-01-31T03:47:29")
        self.assertTrue(self.item[0].get("body_html").startswith("<p>(Adds comments, detail)</p>"))
        self.assertNotIn("description_text", self.item[0])
        self.assertNotIn("archive_description", self.item[0])
        self.assertEqual(self.item[0].get("word_count"), 348)

    def test_can_parse(self):
        self.assertTrue(NewsMLTwoFeedParser().can_parse(self.xml.getroot()))


class ResutersResultsTestCase(BaseNewMLTwoTestCase):
    filename = "tag:reuters.com,0000:newsml_ISS149709:1618095828"

    def test_results(self):
        self.assertTrue(
            self.item[0]
            .get("body_html")
            .startswith(
                "<p>Jan 30 (Gracenote) - Results and standings from the Turkish championship matches on Monday <br/>"
            )
        )
        self.assertNotIn("word_count", self.item[0])


class ANSATestCase(BaseNewMLTwoTestCase):
    filename = "ansa-newsmlg2-text.xml"

    def test_language(self):
        self.assertEqual("it", self.item[0]["language"])


class ReutersOptaTestCase(BaseNewMLTwoTestCase):
    filename = "tag:reuters.com,2018:newsml_MTZXEE13ZXCZES:2"

    def test_body(self):
        self.assertTrue(
            self.item[0].get("body_html").startswith("<pre>Jan 3 (OPTA) - Results and fixtures for the " "Primeira")
        )


class IPTCExampleTextTestCase(BaseNewMLTwoTestCase):
    filename = "LISTING 1 A NewsML-G2 News Item.xml"

    def test_news_item_parsing(self):
        self.assertEqual(1, len(self.item))
        item = self.item[0]
        self.assertEqual("urn:newsml:acmenews.com:20161018:US-FINANCE-FED", item["uri"])
        self.assertEqual("2016-10-21T16:25:32-05:00", item["versioncreated"].isoformat())
        self.assertIn("STRICTLY EMBARGOED", item["ednote"])
        self.assertEqual(1, len(item["authors"]))
        self.assertEqual("2016-10-23T12:00:00+00:00", item["embargoed"].isoformat())

    def test_can_parse(self):
        self.assertTrue(self.parser.can_parse(self.xml.getroot()))


class IPTCExamplePackage(BaseNewMLTwoTestCase):
    filename = "LISTING 6 Simple NewsML-G2 Package.xml"

    def test_news_item_parsing(self):
        self.assertEqual(1, len(self.item))

    def test_can_parse(self):
        self.assertTrue(self.parser.can_parse(self.xml.getroot()))


class ANSACultureParser(BaseNewMLTwoTestCase):
    filename = "ansa_culture.xml"

    def test_content_parsing(self):
        self.assertIn("(ANSA) - ROMA, 1 LUG - TEST FROM XAWES", self.item[0]["body_html"])
        self.assertNotIn("urgency", self.item[0])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from . import unit_base
from decimal import Decimal
from flask_babel import lazy_gettext

METERS_SYMBOL = "metres"


def convert(yards, precision=0):
    """
    Converts from yards to metres
    :param yards: Yards value in string
    :param precision: number of decimal points (int)
    :return: Meters value in string
    """
    yard_to_meter_rate = Decimal(0.9144)
    yards_list = yards.split("-")
    metres = [unit_base.format_converted((Decimal(y) * yard_to_meter_rate), precision) for y in yards_list]
    return "-".join(metres), METERS_SYMBOL


def yards_to_metric(item, **kwargs):
    """Converts distance values from yard to meters"""

    regex = r"(\d+-?,?\.?\d*)((\s*)|(-))((yd)|([yY]ards?))\b"
    return unit_base.do_conversion(item, convert, unit_base.format_output, regex, match_index=0, value_index=1)


name = "yards_to_metric"
label = lazy_gettext("Length yards to metres")
callback = yards_to_metric
access_type = "frontend"
action_type = "interactive"
group = lazy_gettext("length")



from superdesk.tests import TestCase
from superdesk import es_utils


class ESUtilsTestCase(TestCase):
    def test_filter2query(self):
        """Check that a saved_searches style filter is converted correctly to Elastic Search DSL"""
        filter_ = {"query": {"spike": "exclude", "notgenre": '["Article (news)"]'}}
        expected = {
            "query": {
                "bool": {"must": [], "must_not": [{"term": {"state": "spiked"}}, {"term": {"package_type": "takes"}}]}
            },
            "post_filter": {"bool": {"must": [], "must_not": [{"terms": {"genre.name": ["Article (news)"]}}]}},
            "sort": {"versioncreated": "desc"},
            "size": 10,
        }

        with self.app.app_context():
            query = es_utils.filter2query(filter_)
        self.assertEqual(query, expected)

    def test_filter2query_date(self):
        """Check that date a converted correctly to Elastic Search DSL"""
        filter_ = {
            "query": {
                "spike": "exclude",
                "firstcreatedfrom": "now-1M/M",
                "firstcreatedto": "now-1M/M",
                "firstpublished": "last_day",
                "versioncreatedfrom": "01/02/2018",
                "versioncreatedto": "11/12/2018",
            }
        }
        expected = {
            "query": {
                "bool": {"must": [], "must_not": [{"term": {"state": "spiked"}}, {"term": {"package_type": "takes"}}]}
            },
            "post_filter": {
                "bool": {
                    "must": [
                        {
                            "range": {
                                "firstcreated": {"lte": "now-1M/M", "gte": "now-1M/M"},
                                "versioncreated": {
                                    "lte": "2018-12-11T00:00:00+01:00",
                                    "gte": "2018-02-01T23:59:59.999999+01:00",
                                },
                                "firstpublished": {"lte": "now-1d/d", "gte": "now-1d/d"},
                            }
                        }
                    ],
                    "must_not": [],
                }
            },
            "sort": {"versioncreated": "desc"},
            "size": 10,
        }
        with self.app.app_context():
            query = es_utils.filter2query(filter_)
        self.assertEqual(query, expected)

    def test_filter2query_ingest_provider(self):
        """Check that ingest provider is handler correctly"""
        filter_ = {
            "query": {
                "repo": "ingest",
                "ingest_provider": "5c505c8f0d6f137d69cebc99",
                "spike": "exclude",
                "params": "{}",
            }
        }

        expected = {"bool": {"must": [{"term": {"ingest_provider": "5c505c8f0d6f137d69cebc99"}}], "must_not": []}}

        with self.app.app_context():
            query = es_utils.filter2query(filter_)

        self.assertEqual(query["post_filter"], expected)

    def test_filter2query_raw(self):
        filter_ = {
            "query": {
                "spike": "exclude",
                "raw": "headline:test",
            },
        }

        with self.app.app_context():
            query = es_utils.filter2query(filter_)

        self.assertIn(
            {
                "query_string": {
                    "query": "headline:test",
                    "lenient": False,
                    "default_operator": "AND",
                },
            },
            query["query"]["bool"]["must"],
        )

    def test_filter2query_priority(self):
        filter_ = {
            "query": {
                "priority": ["2"],
            },
        }

        with self.app.app_context():
            query = es_utils.filter2query(filter_)

        self.assertIn(
            {
                "terms": {
                    "priority": ["2"],
                },
            },
            query["post_filter"]["bool"]["must"],
        )

    def test_filter2repos_get_types(self):
        repos = es_utils.filter2repos({})
        self.assertIsNone(repos)
        types = es_utils.get_doc_types(repos)
        self.assertEqual(es_utils.REPOS, types)

        repos = es_utils.filter2repos({"query": {"repo": "ingest,published"}})
        self.assertEqual("ingest,published", repos)
        types = es_utils.get_doc_types(repos)
        self.assertEqual(["ingest", "published"], types)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import bson
from superdesk.tests import TestCase
from superdesk import get_resource_service
from superdesk.commands.update_archived_document import UpdateArchivedDocumentCommand


class UpdateArchivedDocumentTestCase(TestCase):
    def setUp(self):
        self.guid = "urn:newsml:localhost:2016-09-12T12:11:40.160498:7237e59f-c42d-4865-aee5-e364aeb2966a"

        self.archived_only_data = [
            {
                "_id": bson.ObjectId("588c1b901d41c805dce70df0"),
                "guid": self.guid,
                "item_id": self.guid,
                "_current_version": 3,
                "type": "text",
                "abstract": "test",
                "state": "fetched",
                "slugline": "slugline",
                "headline": "headline",
                "flags": {"marked_archived_only": True},
                "subject": [{"qcode": "17004000", "name": "Statistics"}],
                "body_html": "Test Document body",
                "source": "AAP",
                "linked_in_packages": [
                    {
                        "package": "urn:newsml:localhost:2017-01-28T15:28:33.535974:30c01757-cfb5-4985-a8f4-c3ecb253a244",
                        "package_type": "takes",
                    }
                ],
            },
            {
                "_id": bson.ObjectId("57d224de069b7f038e9d2a53"),
                "guid": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                "item_id": "urn:newsml:localhost:2016-08-30T06:15:35.379754:3d68cc4c-1f16-4a7f-bfca-92ca50bcdd8f",
                "_current_version": 4,
                "type": "text",
                "abstract": "test",
                "state": "fetched",
                "slugline": "slugline",
                "headline": "headline",
                "flags": {"marked_archived_only": True},
                "subject": [{"qcode": "17004000", "name": "Statistics"}],
                "body_html": "Test Document body",
                "source": "AAP",
            },
        ]

        self.archivedService = get_resource_service("archived")

    def test_update_source(self):
        self.archivedService.post(self.archived_only_data)
        UpdateArchivedDocumentCommand().run("['588c1b901d41c805dce70df0']", "source", "NTB")

        item = self.archivedService.get(req=None, lookup={"_id": "588c1b901d41c805dce70df0"})
        self.assertEqual("NTB", item[0].get("source"))

    def test_update_anpa_category_for_multiple_document(self):
        self.archivedService.post(self.archived_only_data)
        UpdateArchivedDocumentCommand().run(
            "['588c1b901d41c805dce70df0', '57d224de069b7f038e9d2a53']",
            "anpa_category",
            '[{"scheme":null,"qcode":"f","subject":"04000000","name":"Finance"}]',
            True,
        )

        item = self.archivedService.get(req=None, lookup={"_id": "588c1b901d41c805dce70df0"})
        self.assertEqual("f", item[0].get("anpa_category")[0].get("qcode"))
        item = self.archivedService.get(req=None, lookup={"_id": "57d224de069b7f038e9d2a53"})
        self.assertEqual("f", item[0].get("anpa_category")[0].get("qcode"))



import unittest
from unittest import mock
from copy import deepcopy
from datetime import datetime

from apps.rules.routing_rules import RoutingRuleSchemeService
from superdesk.errors import SuperdeskApiError


class RoutingRuleSchemeServiceTest(unittest.TestCase):
    """Base class for RoutingRuleSchemeService tests."""

    def setUp(self):
        self.instance = RoutingRuleSchemeService()


@mock.patch("apps.rules.routing_rules" ".RoutingRuleSchemeService._validate_routing_scheme")
@mock.patch("apps.rules.routing_rules" ".RoutingRuleSchemeService._check_if_rule_name_is_unique")
class OnCreateMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the on_create() method."""

    def test_does_not_modify_semantically_non_empty_schedules(self, *mocks):
        routing_schemes = [
            {
                "name": "scheme_1",
                "rules": [
                    {
                        "name": "rule_1",
                        "schedule": {
                            "day_of_week": ["MON"],
                            "hour_of_day_from": "08:00:00",
                            "hour_of_day_to": "18:00:00",
                            "time_zone": "UTC",
                        },
                    }
                ],
            }
        ]
        original_scheme = deepcopy(routing_schemes[0])
        self.instance.on_create(routing_schemes)
        self.assertEqual(routing_schemes[0], original_scheme)

    def test_does_not_modify_empty_schedules(self, *mocks):
        routing_schemes = [{"name": "scheme_1", "rules": [{"name": "rule_1", "schedule": None}]}]
        original_scheme = deepcopy(routing_schemes[0])
        self.instance.on_create(routing_schemes)
        self.assertEqual(routing_schemes[0], original_scheme)

    def test_sets_semantically_empty_schedules_to_none(self, *mocks):
        routing_schemes = [
            {
                "name": "scheme_1",
                "rules": [{"name": "rule_1", "schedule": {"time_zone": "UTC"}}],  # effectively empty schedule
            }
        ]

        expected_scheme = deepcopy(routing_schemes[0])
        expected_scheme["rules"][0]["schedule"] = None

        self.instance.on_create(routing_schemes)

        self.assertEqual(routing_schemes[0], expected_scheme)


@mock.patch("apps.rules.routing_rules" ".RoutingRuleSchemeService._validate_routing_scheme")
@mock.patch("apps.rules.routing_rules" ".RoutingRuleSchemeService._check_if_rule_name_is_unique")
class OnUpdateMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the on_update() method."""

    def test_does_not_modify_semantically_non_empty_schedules(self, *mocks):
        routing_scheme = {
            "name": "scheme_1",
            "rules": [
                {
                    "name": "rule_1",
                    "schedule": {
                        "day_of_week": ["MON"],
                        "hour_of_day_from": "08:00:00",
                        "hour_of_day_to": "18:00:00",
                        "time_zone": "UTC",
                    },
                }
            ],
        }
        original_scheme = deepcopy(routing_scheme)
        self.instance.on_update(routing_scheme, {})
        self.assertEqual(routing_scheme, original_scheme)

    def test_does_not_modify_empty_schedules(self, *mocks):
        routing_scheme = {"name": "scheme_1", "rules": [{"name": "rule_1", "schedule": None}]}
        original_scheme = deepcopy(routing_scheme)
        self.instance.on_update(routing_scheme, {})
        self.assertEqual(routing_scheme, original_scheme)

    def test_sets_semantically_empty_schedules_to_none(self, *mocks):
        routing_scheme = {
            "name": "scheme_1",
            "rules": [{"name": "rule_1", "schedule": {"time_zone": "UTC"}}],  # effectively empty schedule
        }

        expected_scheme = deepcopy(routing_scheme)
        expected_scheme["rules"][0]["schedule"] = None

        self.instance.on_update(routing_scheme, {})

        self.assertEqual(routing_scheme, expected_scheme)


class ValidateScheduleMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the _validate_schedule() method."""

    def setUp(self):
        super().setUp()
        self.schedule = {"day_of_week": ["WED", "FRI"]}

    @mock.patch("apps.rules.routing_rules.all_timezones_set", {"Foo/Bar", "Foo/Baz", "Here/There"})
    def test_raises_error_on_unknown_time_zone(self):
        self.schedule["time_zone"] = "Invalid/Zone"

        with self.assertRaises(SuperdeskApiError) as context:
            self.instance._validate_schedule(self.schedule)

        # also check the error message
        msg = context.exception.message
        msg = msg.lower() if msg else ""
        self.assertIn("time zone", msg)
        self.assertIn("invalid/zone", msg)

    def test_allows_empty_end_time(self):
        self.schedule["hour_of_day_from"] = "10:30:00"
        self.schedule["hour_of_day_to"] = ""

        try:
            self.instance._validate_schedule(self.schedule)
        except Exception as ex:
            self.fail("Unexpected exception on empty hour_of_day_to: {}".format(ex))


class GetScheduledRoutingRulesMethodTestCase(RoutingRuleSchemeServiceTest):
    """Tests for the _get_scheduled_routing_rules() method."""

    def test_returns_only_the_rules_scheduled_at_current_time(self):
        rules = [
            {
                "name": "rule_1",
                "schedule": {
                    "day_of_week": ["SAT"],
                    "hour_of_day_from": "01:00:00",
                    "hour_of_day_to": "23:00:00",
                    "time_zone": "UTC",
                },
            },
            {
                "name": "rule_2",
                "schedule": {
                    "day_of_week": ["MON", "TUE", "WED"],
                    "hour_of_day_from": "21:00:00",
                    "hour_of_day_to": "22:00:00",
                    "time_zone": "UTC",
                },
            },
            {
                "name": "rule_3",
                "schedule": {
                    "day_of_week": ["TUE"],
                    "hour_of_day_from": "22:00:00",
                    "hour_of_day_to": "23:00:00",
                    "time_zone": "UTC",
                },
            },
        ]

        now = datetime(2015, 8, 18, 21, 30)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], rules[1])

    def test_takes_rule_time_zone_into_account(self):
        rules = [
            {
                "name": "rule_berlin",
                "schedule": {
                    "day_of_week": ["TUE"],
                    "hour_of_day_from": "14:00:00",
                    "hour_of_day_to": "15:00:00",
                    "time_zone": "Europe/Berlin",  # UTC +01:00/+02:00
                },
            },
            {
                "name": "rule_singapore",
                "schedule": {
                    "day_of_week": ["TUE"],
                    "hour_of_day_from": "22:00:00",
                    "hour_of_day_to": "23:00:00",
                    "time_zone": "Asia/Singapore",  # always UTC +08:00
                },
            },
        ]

        now = datetime(2015, 9, 15, 14, 30)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], rules[1])

    def test_assumes_utc_time_zone_if_none_set(self):
        rules = [
            {
                "name": "rule_berlin",
                "schedule": {
                    "day_of_week": ["MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"],
                    "hour_of_day_from": "14:50:00",
                    "hour_of_day_to": "14:55:00",
                    "time_zone": None,
                },
            }
        ]

        now = datetime(2015, 9, 15, 14, 52)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(len(result), 1)
        self.assertEqual(result[0], rules[0])

    def test_assumes_from_the_start_of_the_day_if_start_time_not_set(self):
        rules = [
            {
                "name": "rule_berlin",
                "schedule": {
                    "day_of_week": ["MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"],
                    "hour_of_day_from": None,
                    "hour_of_day_to": "20:30:00",
                    "time_zone": "UTC",
                },
            }
        ]
        now = datetime(2015, 9, 15, 0, 0, 0, 0)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(result, rules)

    def test_assumes_until_the_end_of_the_day_if_end_time_not_set(self):
        rules = [
            {
                "name": "rule_berlin",
                "schedule": {
                    "day_of_week": ["MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"],
                    "hour_of_day_from": "18:45:00",
                    "hour_of_day_to": None,
                    "time_zone": "UTC",
                },
            }
        ]
        now = datetime(2015, 9, 15, 23, 59, 59, 999999)  # Tuesday

        result = self.instance._get_scheduled_routing_rules(rules, now)

        self.assertEqual(result, rules)

    def test_schedule_routing_rules_at_end_time(self):
        rules = [
            {
                "name": "rule_berlin",
                "schedule": {
                    "day_of_week": ["MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"],
                    "hour_of_day_from": "00:00:00",
                    "hour_of_day_to": "23:55:00",
                    "time_zone": "UTC",
                },
            }
        ]
        # before end time
        now = datetime(2015, 9, 15, 23, 54, 59, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, rules)

        # one second after end time
        now = datetime(2015, 9, 15, 23, 55, 1, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, rules)

        # after end time
        now = datetime(2015, 9, 15, 23, 55, 59, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, rules)

        # one second after end time
        now = datetime(2015, 9, 15, 23, 56, 0, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, [])

        # one second after end time
        now = datetime(2015, 9, 15, 23, 56, 1)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, [])

        # after end time
        now = datetime(2015, 9, 15, 23, 59, 59, 999999)  # Tuesday
        result = self.instance._get_scheduled_routing_rules(rules, now)
        self.assertEqual(result, [])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import app_models
from .base_model import BaseModel


def register_model(model):
    """Register a model - to be used in bootstrap.

    :param model: object The model object.
    """
    assert isinstance(model, BaseModel), "Invalid model %s" % model
    app_models[model.name()] = model


def get_model(name):
    """Return an instance of the model identified by given name."""
    if isinstance(name, BaseModel) or (type(name) is type and issubclass(name, BaseModel)):
        name = name.name()
    return app_models[name]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import current_app as app, json, g
from eve_elastic.elastic import set_filters
from copy import deepcopy

import superdesk

from superdesk import get_resource_service
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE, get_schema
from superdesk.metadata.utils import aggregations as common_aggregations, item_url, get_elastic_highlight_query
from apps.archive.archive import SOURCE as ARCHIVE
from superdesk.resource import build_custom_hateoas
from apps.publish.published_item import published_item_fields
from superdesk import es_utils


class SearchService(superdesk.Service):
    """Federated search service.

    It can search against different collections like Ingest, Production, Archived etc.. at the same time.
    """

    repos = None
    aggregations = deepcopy(common_aggregations)

    @property
    def elastic(self):
        return app.data.elastic

    def __init__(self, datasource, backend):
        super().__init__(datasource=datasource, backend=backend)

    def get_ingest_filters(self):
        """
        Returns Ingest filters
        """
        return [{"term": {"_type": "ingest"}}]

    def get_archive_filters(self):
        """
        Returns Archive filters filters

        If the content state is draft, it must be from the current user
        """
        user_id = (g.get("user") or {}).get("_id")
        return [
            {"exists": {"field": "task.desk"}},
            {
                "bool": {
                    "should": [
                        {"and": [{"term": {ITEM_STATE: CONTENT_STATE.DRAFT}}, {"term": {"task.user": str(user_id)}}]},
                        {
                            "terms": {
                                ITEM_STATE: [
                                    CONTENT_STATE.FETCHED,
                                    CONTENT_STATE.ROUTED,
                                    CONTENT_STATE.PROGRESS,
                                    CONTENT_STATE.SUBMITTED,
                                    CONTENT_STATE.SPIKED,
                                    CONTENT_STATE.CORRECTION,
                                ]
                            }
                        },
                    ],
                    "must_not": {"term": {"version": 0}},
                    "minimum_should_match": 1,
                }
            },
        ]

    def get_published_filters(self):
        """
        Returns published filters
        """
        return [
            {"term": {"_type": "published"}},
            {
                "terms": {
                    ITEM_STATE: [
                        CONTENT_STATE.SCHEDULED,
                        CONTENT_STATE.PUBLISHED,
                        CONTENT_STATE.KILLED,
                        CONTENT_STATE.RECALLED,
                        CONTENT_STATE.CORRECTED,
                        CONTENT_STATE.UNPUBLISHED,
                        CONTENT_STATE.BEING_CORRECTED,
                    ]
                }
            },
        ]

    def get_archived_filters(self):
        """
        Returns archived filters
        """
        return [{"term": {"_type": "archived"}}]

    def _get_private_filters(self, repo, invisible_stages):
        query = {"and": []}

        if repo == "ingest":
            query["and"].extend(self.get_ingest_filters())
        elif repo == "archive":
            query["and"].extend(self.get_archive_filters())
        elif repo == "published":
            query["and"].extend(self.get_published_filters())
        elif repo == "archived":
            query["and"].extend(self.get_archived_filters())

        if invisible_stages and repo != "ingest":
            query["and"].append({"not": {"terms": {"task.stage": invisible_stages}}})

        return query

    def _get_query(self, req):
        """Get elastic query."""
        args = getattr(req, "args", {})
        query = json.loads(args.get("source")) if args.get("source") else {"query": {"filtered": {}}}
        if app.data.elastic.should_aggregate(req):
            query["aggs"] = self.aggregations

        if app.data.elastic.should_highlight(req):
            highlight_query = get_elastic_highlight_query(self._get_highlight_query_string(req))
            if highlight_query:
                query["highlight"] = highlight_query
        return query

    def _get_highlight_query_string(self, req):
        args = getattr(req, "args", {})
        source = json.loads(args.get("source")) if args.get("source") else {"query": {"filtered": {}}}
        query_string = source.get("query", {}).get("filtered", {}).get("query", {}).get("query_string")
        return query_string

    def _get_projected_fields(self, req):
        """Get elastic projected fields."""
        if app.data.elastic.should_project(req):
            return app.data.elastic.get_projected_fields(req)

    def _get_types(self, req):
        """Get document types for the given query."""
        args = getattr(req, "args", {})
        repos = args.get("repo")
        return es_utils.get_doc_types(repos, self.repos)

    def _get_filters(self, repos, invisible_stages):
        """
        Gets filters for the passed repos.
        """
        filters = []

        for repo in repos:
            filters.append(self._get_private_filters(repo, invisible_stages))

        return [{"or": filters}] if filters else []

    def get_stages_to_exclude(self):
        """
        Returns the list of the current users invisible stages
        """
        user = g.get("user", {})
        if "invisible_stages" in user:
            stages = user.get("invisible_stages")
        else:
            stages = get_resource_service("users").get_invisible_stages_ids(user.get("_id"))

        return stages

    def get(self, req, lookup):
        """
        Runs elastic search on multiple doc types.
        """

        query = self._get_query(req)
        fields = self._get_projected_fields(req)
        types = self._get_types(req)
        excluded_stages = self.get_stages_to_exclude()
        filters = self._get_filters(types, excluded_stages)

        # if the system has a setting value for the maximum search depth then apply the filter
        if not app.settings["MAX_SEARCH_DEPTH"] == -1:
            query["terminate_after"] = app.settings["MAX_SEARCH_DEPTH"]

        if filters:
            set_filters(query, filters)

        params = {}
        if fields:
            params["_source"] = fields

        docs = self.elastic.search(query, types, params)

        for resource in types:
            response = {app.config["ITEMS"]: [doc for doc in docs if doc["_type"] == resource]}
            getattr(app, "on_fetched_resource")(resource, response)
            getattr(app, "on_fetched_resource_%s" % resource)(response)

        return docs

    def _get_docs(self, hits):
        """Parse hits from elastic and return only docs.

        This will remove some extra metadata from elastic.

        :param hits: elastic hits dictionary
        """
        return self.elastic._parse_hits(hits, "ingest")  # any resource with item schema will do

    def find_one(self, req, **lookup):
        """Find item by id in all collections."""
        _id = lookup["_id"]
        for resource in self._get_types(req):
            id_field = "item_id" if resource == "published" else "_id"
            resource_lookup = {id_field: _id}
            item = get_resource_service(resource).find_one(req=req, **resource_lookup)
            if item:
                return item

    def on_fetched(self, doc):
        """
        Overriding to add HATEOS for each individual item in the response.

        :param doc: response doc
        :type doc: dict
        """

        docs = doc[app.config["ITEMS"]]
        for item in docs:
            build_custom_hateoas({"self": {"title": item["_type"], "href": "/{}/{{_id}}".format(item["_type"])}}, item)

    def get_available_indexes(self):
        """Returns a set of the configured indexes

        :return:
        """
        return set(es_utils.get_index().split(","))


class SearchResource(superdesk.Resource):
    resource_methods = ["GET"]
    item_methods = ["GET"]
    item_url = item_url
    schema = get_schema(versioning=True)
    schema.update(published_item_fields)
    datasource = {"projection": {field: 1 for field in list(schema.keys()) + ["archive_item"]}}


def init_app(app):
    search_service = SearchService(ARCHIVE, backend=superdesk.get_backend())
    SearchResource("search", app=app, service=search_service)

    # Set the start of week config for use in both server and client
    app.client_config["start_of_week"] = app.config.get("START_OF_WEEK") or 0



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from apps import auth

from .users import UsersResource
from .services import UsersService, DBUsersService, is_admin  # noqa


def init_app(app):
    endpoint_name = "users"
    service = DBUsersService(endpoint_name, backend=superdesk.get_backend())
    UsersResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name="users", label="User Management", description="User can manage users.")

    # Registering with intrinsic privileges because: A user should be allowed to update their own profile.
    superdesk.intrinsic_privilege(resource_name="users", method=["PATCH"])

    app.client_config.setdefault("user", {}).update(
        {
            "username_pattern": app.config.get("USER_USERNAME_PATTERN"),
        }
    )


def get_user_from_request(required=False):
    """
    Get user authenticated for current request.

    :param boolean required: if True and there is no user it will raise an error
    """

    return auth.get_user(required)
